Package: ontoProc2
Date: 2026-07-13
Title: Ontology Facilities Based on INCAtools Semantic SQL
Version: 0.99.30
Authors@R: 
    c(person(
        "Vincent", "Carey", role = c("aut", "cre"),
        email = "stvjc@channing.harvard.edu",
        comment = c(ORCID = "0000-0003-4046-0063")),
      person("Anthropic-Claude", "Sonnet4.6", role="eng"),
      person(
        "NIH NHGRI 2U24HG004059-17",
        role = "fnd"
      )
     )
Description: This package provides ontology facilities based on 
	INCAtools Semantic SQL.  Tooling is provided to retrieve
	and cache SQLite databases representing curated ontologies.
	ontologyIndex ontology_index instances can be produced.
	Anthropic's Claude was used in the development of S7
	classes and methods and in the production of documentation.
License: Artistic-2.0
Encoding: UTF-8
Depends: R (>= 4.5.0)
Imports: RSQLite, DBI, BiocFileCache, R.utils, dplyr, ontologyIndex,
        ontologyPlot, graph, Rgraphviz, methods, S7, utils, httr2,
        xml2, digest
Suggests: knitr, testthat, BiocStyle, DT, GO.db, AnnotationDbi, stringr
VignetteBuilder: knitr
biocViews: Infrastructure, DataRepresentation, Pathways, SingleCell
URL: https://github.com/vjcitn/ontoProc2
BugReports: https://github.com/vjcitn/ontoProc2/issues
Config/roxygen2/version: 8.0.0
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/ontoProc2
git_branch: devel
git_last_commit: 1d4df64
git_last_commit_date: 2026-07-28
Repository: Bioconductor 3.24
Date/Publication: 2026-07-29
NeedsCompilation: no
Packaged: 2026-07-29 23:17:53 UTC; biocbuild
Author: Vincent Carey [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4046-0063>),
  Anthropic-Claude Sonnet4.6 [eng],
  NIH NHGRI 2U24HG004059-17 [fnd]
Maintainer: Vincent Carey <stvjc@channing.harvard.edu>

# Generated by roxygen2: do not edit by hand

export(PREDICATES)
export(SemsqlConn)
export(bbop_sqlite_db_gz)
export(cells_with_pmp)
export(count_by_prefix)
export(count_descendants)
export(describe_table)
export(disconnect)
export(find_by_restriction)
export(find_intersection)
export(get_ancestors)
export(get_ancestors_partonomy)
export(get_definition)
export(get_descendants)
export(get_descendants_partonomy)
export(get_direct_edges)
export(get_direct_subclasses)
export(get_direct_superclasses)
export(get_label)
export(get_prefix)
export(get_present_pmp)
export(get_restrictions)
export(get_synonyms)
export(get_term_info)
export(improve_nodes)
export(is_connected)
export(list_tables)
export(make_graphNEL_from_ontology_plot)
export(onto_plot2)
export(reconnect)
export(report)
export(retrieve_semsql_conn)
export(run_query)
export(search_labels)
export(semsql_connect)
export(semsql_to_oi)
export(semsql_url)
export(with_connection)
import(BiocFileCache)
import(DBI)
import(RSQLite)
import(Rgraphviz)
import(S7)
import(graph)
importFrom(R.utils,gunzip)
importFrom(methods,as)
importFrom(methods,new)
importFrom(ontologyIndex,ontology_index)
importFrom(ontologyPlot,onto_plot)
importFrom(utils,data)
importFrom(utils,download.file)

# ontoProc2

This package constitutes a "second generation" approach to
management and use of ontologies in Bioconductor.

The [INCAtools Semantic SQL project](https://github.com/INCATools/semantic-sql) ([tutorial notebook](https://github.com/INCATools/semantic-sql/blob/main/notebooks/SemanticSQL-Tutorial.ipynb)) identifies an AWS S3 bucket where a large number of ontologies
are available.

```
%vjc> aws s3 ls s3://bbop-sqlite/ |grep db.gz
2023-09-02 17:38:48    4268563 ado.db.gz
2023-09-02 17:38:48   11841154 agro.db.gz
2023-09-02 17:38:48     301889 aio.db.gz
2023-09-02 17:38:48   42842146 aism.db.gz
2023-09-02 17:38:48     294887 amphx.db.gz
2023-09-02 17:38:48     303328 apo.db.gz
2023-09-02 17:38:48    3457483 apollo_sv.db.gz
2023-09-02 17:38:48    7245916 aro.db.gz
2023-09-02 17:38:48    5939737 bao.db.gz
2023-09-02 17:38:48     767913 bcio.db.gz
2023-09-02 17:38:50     552497 bco.db.gz
2023-09-02 17:38:51 1600927276 bero.db.gz
2023-09-02 17:38:51     108284 bfo.db.gz
2023-09-02 17:38:54     575935 biolink.db.gz
2023-09-02 17:38:55        646 biopax.db.gz
2023-09-02 17:38:55     463088 biovoices.db.gz
...
```
In this package we will provide tools to retrieve, cache, and make use of these ontologies.

## Installation

For Bioconductor versions 3.24 and beyond, use `BiocManager::install("ontoProc2")`.  Otherwise,
use `BiocManager::install("vjcitn/ontoProc2")`.

# =============================================================================
# SemanticSQL R Interface - S7 Class Implementation
# =============================================================================

#' SemsqlConn: S7 connection wrapper for SemanticSQL databases
#'
#' @description
#' An S7 class that encapsulates a SQLite connection to an ontology database
#' following the SemanticSQL schema. Provides methods for common ontology
#' queries including label lookup, ancestor/descendant traversal, and
#' relationship queries.
#'
#' Properties:
#' \describe{
#'   \item{con}{The DBI connection object (SQLiteConnection)}
#'   \item{db_path}{character(1) path to the SQLite database file}
#'   \item{ontology_prefix}{character(1) primary ontology prefix,
#'     e.g. `"CL"` for the Cell Ontology}
#' }
#' @import S7
#' @param con DBI connection object
#' @param db_path character path to SQLite database file
#' @param ontology_prefix character, e.g., 'CL' for cell ontology
#' @return wrapped connection
#' @export
SemsqlConn <- new_class(
  "SemsqlConn",
  properties = list(
    con = class_any,
    db_path = class_character,
    ontology_prefix = class_character
  ),
  validator = function(self) {
    if (!inherits(self@con, "SQLiteConnection")) {
      return("@con must be a SQLiteConnection object")
    }
    if (!dbIsValid(self@con)) {
      return("Database connection is not valid")
    }
    NULL
  }
)
S4_register(SemsqlConn)

# =============================================================================
# CONSTRUCTOR
# =============================================================================

#' Create a SemsqlConn connection
#'
#' @description
#' Opens a connection to a SemanticSQL SQLite database, either by supplying
#' a direct file path or by referencing a short ontology name that is
#' retrieved and cached via `BiocFileCache`.
#'
#' @param db_path character(1) or NULL. Path to an existing SQLite database
#'   file. Either `db_path` or `ontology` must be supplied.
#' @param ontology_prefix character(1) or NULL. Primary CURIE prefix for the
#'   ontology (e.g. `"CL"`). If NULL and `ontology` is supplied,
#'   defaults to `toupper(ontology)`; otherwise auto-detected from the
#'   database.
#' @param ontology character(1) or NULL. Short name of an INCAtools ontology
#'   (e.g. `"cl"`, `"go"`). If supplied,
#'   [retrieve_semsql_conn()] is called to locate or download the
#'   cached database.
#' @param cache a `BiocFileCache` instance used when `ontology` is
#'   supplied. Defaults to `BiocFileCache::BiocFileCache()`.
#' @param validate logical(1) if TRUE (the default value) the ontology code
#' is checked against available Semantic SQL resources at INCAtools.  Set to
#' FALSE if using off line.
#' @param ... passed to [retrieve_semsql_conn()] and ultimately
#'   to [utils::download.file()].
#' @note The connection has flag `SQLITE_RO` for read-only access.  There will
#' be an attempt to validate the `ontology` tag that is supplied, against all
#' available Semantic SQL resources available at INCAtools bucket.  Function fails
#' if a match cannot be made, which in general requires network access.
#' @return A [SemsqlConn()] object.
#' @examples
#' # by ontology short name (downloads if not cached)
#' goref <- semsql_connect(ontology = "go")
#' goref
#' disconnect(goref)
#' @export
semsql_connect <- function(db_path = NULL, ontology_prefix = NULL,
                           ontology = NULL,
                           cache = BiocFileCache::BiocFileCache(), validate = TRUE, ...) {
  if (!is.null(ontology)) {
    if (validate) {
      candidates = suppressMessages({bbop_sqlite_db_gz(bfc=cache)})
      prefs = gsub(".db.gz", "", candidates, fixed = TRUE)
      if (!(ontology %in% prefs)) stop(sprintf("Ontology code '%s' not found as a prefix to .db.gz\n", ontology),
          "  in the output of bbop_sqlite_db_gz().  Please check\n",
          "  results of bbop_sqlite_db_gz() for a valid ontology code.\n")
      }
    raw_con <- retrieve_semsql_conn(ontology, cache = cache, ...)
    db_path <- raw_con@dbname
    dbDisconnect(raw_con)
    if (is.null(ontology_prefix)) ontology_prefix <- toupper(ontology)
  }
  if (is.null(db_path)) stop("Supply either db_path or ontology")
  if (!file.exists(db_path)) {
    stop("Database file not found: ", db_path)
  }

  con <- dbConnect(RSQLite::SQLite(), db_path, flags=SQLITE_RO)

  if (is.null(ontology_prefix)) {
    prefix_query <- "
      SELECT SUBSTR(subject, 1, INSTR(subject, ':') - 1) AS prefix,
             COUNT(*) AS n
      FROM rdfs_label_statement
      WHERE subject LIKE '%:%'
      GROUP BY prefix
      ORDER BY n DESC
      LIMIT 1
    "
    result <- dbGetQuery(con, prefix_query)
    ontology_prefix <- if (nrow(result) > 0) result$prefix[1] else ""
  }

  message("Connected to SemanticSQL database: ", db_path)
  message("Primary ontology prefix: ", ontology_prefix)

  SemsqlConn(
    con = con,
    db_path = db_path,
    ontology_prefix = ontology_prefix
  )
}

# =============================================================================
# GENERICS
# =============================================================================

#' Disconnect a SemsqlConn from its database
#'
#' @param x A `SemsqlConn` object.
#' @param quiet logical(1) if TRUE suppresses the disconnection message
#'   (default FALSE).
#' @param ... not used
#' @return The `SemsqlConn` object invisibly.
#' @export
disconnect <- new_generic("disconnect", "x", function(x, quiet=FALSE, ...) S7_dispatch())

#' Test whether a SemsqlConn has a valid open connection
#'
#' @param x A `SemsqlConn` object.
#' @param ... not used
#' @return logical(1).
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' is_connected(goref) # TRUE
#' disconnect(goref)
#' is_connected(goref) # FALSE
#' @export
is_connected <- new_generic("is_connected", "x")

#' List tables in a SemsqlConn database
#'
#' @param x A `SemsqlConn` object.
#' @param ... not used
#' @return character vector of table names.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' list_tables(goref)
#' disconnect(goref)
#' @export
list_tables <- new_generic("list_tables", "x")

#' Describe the columns of a table in a SemsqlConn database
#'
#' @param x A `SemsqlConn` object.
#' @param table_name character(1) name of the table.
#' @param ... not used
#' @return data.frame with PRAGMA table_info output (columns: cid, name, type,
#'   notnull, dflt_value, pk).
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' describe_table(goref, "rdfs_label_statement")
#' disconnect(goref)
#' @export
describe_table <- new_generic("describe_table", "x",
  function(x, table_name, ...) S7_dispatch())

#' Retrieve the ontology prefix from a SemsqlConn
#'
#' @param x A `SemsqlConn` object.
#' @param ... not used
#' @return character(1) the primary ontology prefix (e.g. `"GO"`).
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_prefix(goref)
#' disconnect(goref)
#' @export
get_prefix <- new_generic("get_prefix", "x",
  function(x, ...) S7_dispatch())

#' Search term labels in a SemsqlConn database
#'
#' @param x A `SemsqlConn` object.
#' @param pattern character(1) substring to match against rdfs:label values
#'   (SQL LIKE pattern, case-insensitive on most SQLite builds).
#' @param limit integer(1) maximum number of rows to return (default 20).
#' @return data.frame with columns `subject` and `label`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' search_labels(goref, "apoptosis")
#' disconnect(goref)
#' @export
search_labels <- new_generic("search_labels", "x",
  function(x, pattern, limit=20L) S7_dispatch())

#' Get the rdfs:label for a term
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE, e.g. `"GO:0006915"`.
#' @param ... not used
#' @return character(1) label, or `NA_character_` if not found.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_label(goref, "GO:0006915") # "apoptotic process"
#' disconnect(goref)
#' @export
get_label <- new_generic("get_label", "x",
  function(x, term_id, ...) S7_dispatch())

#' Get the text definition for a term
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @return character(1) definition text, or `NA_character_` if not found.
#' @param ... not used
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_definition(goref, "GO:0006915")
#' disconnect(goref)
#' @export
get_definition <- new_generic("get_definition", "x",
   function(x, term_id, ...) S7_dispatch())

#' Get synonyms for a term
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param type character(1) synonym scope: one of `"all"`,
#'   `"exact"`, `"broad"`, `"narrow"`, `"related"`.
#' @param ... not used
#' @return data.frame with columns `subject`, `predicate`,
#'   `synonym`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_synonyms(goref, "GO:0006915")
#' get_synonyms(goref, "GO:0006915", type = "exact")
#' disconnect(goref)
#' @export
get_synonyms <- new_generic("get_synonyms", "x",
  function(x, term_id, type=c("all", "exact", "broad", "narrow", "related"), ...) S7_dispatch())

#' Retrieve a summary of information about a term
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param ... not used
#' @return list with elements `id`, `label`, `definition`,
#'   `synonyms`, `superclasses`, `subclasses`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' info <- get_term_info(goref, "GO:0006915")
#' info$label
#' info$superclasses
#' disconnect(goref)
#' @export
get_term_info <- new_generic("get_term_info", "x",
  function(x, term_id, ...) S7_dispatch())

#' Get direct edges in the ontology graph for a term
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param direction character(1) one of `"outgoing"`, `"incoming"`,
#'   `"both"`.
#' @param ... not used
#' @return data.frame with columns `subject`, `subject_label`,
#'   `predicate`, `predicate_label`, `object`,
#'   `object_label`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_direct_edges(goref, "GO:0006915")
#' get_direct_edges(goref, "GO:0006915", direction = "both")
#' disconnect(goref)
#' @export
get_direct_edges <- new_generic("get_direct_edges", "x",
   function(x, term_id, direction=c("outgoing", "incoming", "both"), ...) S7_dispatch())

#' Get direct subclasses of a term
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param ... not used
#' @return data.frame with columns `id` and `label`, ordered by
#'   label.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' # direct children of "apoptotic process" (GO:0006915)
#' get_direct_subclasses(goref, "GO:0006915")
#' disconnect(goref)
#' @export
get_direct_subclasses <- new_generic("get_direct_subclasses", "x",
   function(x, term_id, ...) S7_dispatch())

#' Get direct superclasses of a term
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param ... not used
#' @return data.frame with columns `id` and `label`, ordered by
#'   label.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_direct_superclasses(goref, "GO:0006915")
#' disconnect(goref)
#' @export
get_direct_superclasses <- new_generic("get_direct_superclasses", "x",
   function(x, term_id, ...) S7_dispatch())

#' Get all ancestors of a term via entailed edges
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param predicates character vector of predicate CURIEs to follow.
#'   Defaults to `"rdfs:subClassOf"`. See [PREDICATES()] for
#'   common values.
#' @param include_self logical(1) whether to include the term itself
#'   (default `FALSE`).
#' @param ... not used
#' @return data.frame with columns `id`, `label`, `predicate`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_ancestors(goref, "GO:0006915")
#' disconnect(goref)
#' @export
get_ancestors <- new_generic("get_ancestors", "x",
   function(x, term_id, predicates="rdfs:subClassOf", include_self=FALSE, ...) S7_dispatch())

#' Get all descendants of a term via entailed edges
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param predicates character vector of predicate CURIEs to follow.
#'   Defaults to `"rdfs:subClassOf"`.
#' @param include_self logical(1) whether to include the term itself
#'   (default `FALSE`).
#' @param ... not used
#' @return data.frame with columns `id`, `label`, `predicate`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_descendants(goref, "GO:0006915")
#' disconnect(goref)
#' @export
get_descendants <- new_generic("get_descendants", "x",
   function(x, term_id, predicates="rdfs:subClassOf", include_self=FALSE, ...) S7_dispatch())

#' Get OWL someValuesFrom restrictions for a term
#'
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param ... not used
#' @return data.frame with columns `restriction_id`, `property`,
#'   `property_label`, `filler`, `filler_label`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' # mitochondrion (GO:0005739) has part-of restrictions to cell
#' get_restrictions(goref, "GO:0005739")
#' disconnect(goref)
#' @export
get_restrictions <- new_generic("get_restrictions", "x",
  function(x, term_id, ...) S7_dispatch())

#' Find terms that have a given OWL someValuesFrom restriction
#'
#' @param x A `SemsqlConn` object.
#' @param property character(1) property CURIE
#'   (e.g. `"BFO:0000050"` for part-of).
#' @param filler character(1) filler class CURIE.
#' @param include_filler_descendants logical(1) if TRUE also match subclasses
#'   of `filler` (default `FALSE`).
#' @param ... not used
#' @return data.frame with columns `id` and `label`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' # cellular components that are part_of nucleus (GO:0005634)
#' find_by_restriction(goref, "BFO:0000050", "GO:0005634")
#' disconnect(goref)
#' @export
find_by_restriction <- new_generic("find_by_restriction", "x",
  function(x, property, filler, include_filler_descendants=FALSE, ...) S7_dispatch())
# 
#' Find terms that are descendants of a superclass and have a given restriction
#'
#' @param x A `SemsqlConn` object.
#' @param superclass_id character(1) CURIE of the superclass.
#' @param relation_property character(1) property CURIE for the restriction.
#' @param related_to_id character(1) filler CURIE for the restriction.
#' @param ... not used
#' @return data.frame with columns `id` and `label`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' # CC terms (GO:0005575) that are part_of nucleus (GO:0005634)
#' find_intersection(goref, "GO:0005575", "BFO:0000050", "GO:0005634")
#' disconnect(goref)
#' @export
 find_intersection <- new_generic("find_intersection", "x",
     function(x, superclass_id, relation_property, related_to_id, ...) S7_dispatch())
 
#' Count the number of descendants of a term
#' @param x A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param predicate character(1) predicate to traverse
#'   (default `"rdfs:subClassOf"`).
#' @param ... not used
#' @return integer(1).
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' count_descendants(goref, "GO:0006915") # all apoptosis subtypes
#' disconnect(goref)
#' @export
 count_descendants <- new_generic("count_descendants", "x", 
    function(x, term_id, predicate = "rdfs:subClassOf", ...) S7_dispatch())

#' Count labeled terms grouped by CURIE prefix
#' @param x A `SemsqlConn` object.
#' @param ... not used.
#' @return data.frame with columns `prefix` and `n`, ordered by
#'   `n` descending.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' count_by_prefix(goref)
#' disconnect(goref)
#' @export
count_by_prefix <- new_generic("count_by_prefix", "x", function(x, ...) S7_dispatch())


#' Run an arbitrary SQL query against a SemsqlConn database
#'
#' @param x A `SemsqlConn` object.
#' @param sql character(1) SQL query string.
#' @param ... not used
#' @return data.frame with query results.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' run_query(
#'   goref,
#'   "SELECT subject, value AS label FROM rdfs_label_statement LIMIT 5"
#' )
#' disconnect(goref)
#' @export
run_query <- new_generic("run_query", "x", function(x, sql, ...)
   S7_dispatch())

#' Display a detailed report of a SemsqlConn object
#'
#' @description
#' Displays a verbose, formatted representation of a `SemsqlConn` object
#' including connection status, database statistics (labeled terms, edges,
#' definitions), prefix breakdown, and available key tables. More informative
#' than `print()`, intended for interactive exploration.
#'
#' @param object A `SemsqlConn` object.
#' @param ... additional arguments (currently unused)
#' @return The `SemsqlConn` object invisibly.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' report(goref)
#' disconnect(goref)
#' @export
report <- S7::new_generic("report", "object", function(object, ...) {
  S7::S7_dispatch()
})

#' Reconnect a SemsqlConn to its database
#'
#' @description
#' Attempts to reconnect a disconnected `SemsqlConn` object to its
#' database. Returns a new `SemsqlConn`; the original cannot be modified
#' in place due to S7 value semantics.
#'
#' @param x A `SemsqlConn` object.
#' @param ... not used
#' @return A new `SemsqlConn` object with an active connection.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' disconnect(goref)
#' goref <- reconnect(goref)
#' disconnect(goref)
#' @export
reconnect <- new_generic("reconnect", "x")

# =============================================================================
# CONNECTION MANAGEMENT METHODS
# =============================================================================

method(disconnect, SemsqlConn) <- function(x, quiet = FALSE) {
  if (dbIsValid(x@con)) {
    db_name <- basename(x@db_path)
    dbDisconnect(x@con)
    if (!quiet) message("Disconnected from '", db_name, "'")
  } else {
    if (!quiet) message("Connection already closed")
  }
  invisible(x)
}

method(is_connected, SemsqlConn) <- function(x) {
  dbIsValid(x@con)
}

method(list_tables, SemsqlConn) <- function(x) {
  dbListTables(x@con)
}

# constructing SQL query via sprintf here is OK because the 
# inserted entity is validated
method(describe_table, SemsqlConn) <- function(x, table_name) {
  valid_tables <- dbListTables(x@con)
  if (!(table_name %in% valid_tables)) stop("Unknown table: ", table_name)
  query <- sprintf("PRAGMA table_info(%s)", table_name)
  dbGetQuery(x@con, query)
}

method(get_prefix, SemsqlConn) <- function(x) {
  x@ontology_prefix
}

method(run_query, SemsqlConn) <- function(x, sql) {
  dbGetQuery(x@con, sql)
}

method(reconnect, SemsqlConn) <- function(x) {
  if (is_connected(x)) {
    message("Already connected")
    return(x)
  }
  if (!file.exists(x@db_path)) {
    stop("Database file no longer exists: ", x@db_path)
  }
  semsql_connect(x@db_path, x@ontology_prefix)
}

# =============================================================================
# BASIC QUERY METHODS
# =============================================================================


# LESS VULNERABLE BECAUSE DRIVER HANDLES TYPING, NO INJECTION SURFACE
method(search_labels, SemsqlConn) <- function(x, pattern, limit = 20L) {
  dbGetQuery(x@con,
    "SELECT subject, value AS label FROM rdfs_label_statement
      WHERE value LIKE ? LIMIT ?", param = list(paste0("%", pattern, "%"), as.integer(limit)))
}


method(get_label, SemsqlConn) <- function(x, term_id) {
    dbGetQuery(x@con, "SELECT subject, value AS label
         FROM rdfs_label_statement
         WHERE subject = ?",
         param = list(term_id)
         )
}

method(get_definition, SemsqlConn) <- function(x, term_id) {
  query <- "SELECT subject, value AS definition
    FROM has_text_definition_statement
    WHERE subject = ?"
  result <- dbGetQuery(x@con, query, param=list(term_id))
  if (nrow(result) == 0) {
    return(NA_character_)
  }
  result$definition[1]
}

method(get_synonyms, SemsqlConn) <- function(
    x, term_id, type = c("all", "exact", "broad", "narrow", "related")) {

  type <- match.arg(type)

  if (type == "all") {
    # Bypass the has_oio_synonym_statement view — DuckDB cannot parse
    # its UNION definition when reading semsql SQLite attachments.
    # Query statements directly instead.
    sql <- "SELECT subject, predicate, value AS synonym
            FROM statements
            WHERE subject = ?
              AND predicate IN (
                'oio:hasExactSynonym',
                'oio:hasRelatedSynonym',
                'oio:hasNarrowSynonym',
                'oio:hasBroadSynonym'
              )"
    return(dbGetQuery(x@con, sql, params = list(term_id)))
  }

  view_name <- switch(type,
    exact   = "has_exact_synonym_statement",
    broad   = "has_broad_synonym_statement",
    narrow  = "has_narrow_synonym_statement",
    related = "has_related_synonym_statement"
  )
  sql <- sprintf(
    "SELECT subject, predicate, value AS synonym FROM %s WHERE subject = ?",
    view_name
  )
  dbGetQuery(x@con, sql, params = list(term_id))
}



method(get_term_info, SemsqlConn) <- function(x, term_id) {
  list(
    id = term_id,
    label = get_label(x, term_id),
    definition = get_definition(x, term_id),
    synonyms = get_synonyms(x, term_id),
    superclasses = get_direct_superclasses(x, term_id),
    subclasses = get_direct_subclasses(x, term_id)
  )
}

# =============================================================================
# EDGE QUERY METHODS
# =============================================================================

method(get_direct_edges, SemsqlConn) <- function(x, term_id, 
   direction=c("outgoing", "incoming", "both")) {
 
  direction = match.arg(direction)
  query.init <- "SELECT
      e.subject,
      sl.value AS subject_label,
      e.predicate,
      pl.value AS predicate_label,
      e.object,
      ol.value AS object_label
    FROM edge e
    LEFT JOIN rdfs_label_statement sl ON e.subject = sl.subject
    LEFT JOIN rdfs_label_statement pl ON e.predicate = pl.subject
    LEFT JOIN rdfs_label_statement ol ON e.object = ol.subject
    WHERE"
  if (direction == "outgoing") {
       query.fin <- "e.subject = ?"
       query = paste(query.init, query.fin)
       return(dbGetQuery(x@con, query, param=list(term_id)))
       }
  else if (direction == "incoming") {
       query.fin <- "e.object = ?"
       query = paste(query.init, query.fin)
       return(dbGetQuery(x@con, query, param=list(term_id)))
       }
  else if (direction == "both") {
       query.fin <- "e.subject = ? OR e.object = ?"
       query = paste(query.init, query.fin)
       return(dbGetQuery(x@con, query, param=list(term_id, term_id)))
       }
}

method(get_direct_subclasses, SemsqlConn) <- function(x, term_id) {
  query <- "SELECT
      e.subject AS id,
      l.value AS label
    FROM edge e
    LEFT JOIN rdfs_label_statement l ON e.subject = l.subject
    WHERE e.object = ?
      AND e.predicate = 'rdfs:subClassOf'
    ORDER BY l.value"
  dbGetQuery(x@con, query, param=list(term_id))
}

method(get_direct_superclasses, SemsqlConn) <- function(x, term_id) {
  query <- "SELECT
      e.object AS id,
      l.value AS label
    FROM edge e
    LEFT JOIN rdfs_label_statement l ON e.object = l.subject
    WHERE e.subject = ?
      AND e.predicate = 'rdfs:subClassOf'
    ORDER BY l.value"
  dbGetQuery(x@con, query, param=list(term_id))
}

# =============================================================================
# ENTAILED (TRANSITIVE) QUERY METHODS
# =============================================================================

method(get_ancestors, SemsqlConn) <- function(
  x, term_id,
  predicates = "rdfs:subClassOf", include_self = FALSE
) {
  # One ? per predicate — built from length, not from values (safe)
  placeholders <- paste(rep("?", length(predicates)), collapse = ", ")

  sql <- sprintf("
    SELECT
      ee.object AS id,
      l.value AS label,
      ee.predicate
    FROM entailed_edge ee
    LEFT JOIN rdfs_label_statement l ON ee.object = l.subject
    WHERE ee.subject = ?
      AND ee.predicate IN (%s)
    ORDER BY l.value
  ", placeholders)

  # Positional order: term_id fills the first ?, predicates fill the rest
  result <- dbGetQuery(x@con, sql, param = c(list(term_id), as.list(predicates)))

  if (!include_self && nrow(result) > 0) {
    result <- result[result$id != term_id, , drop = FALSE]
  }
  result
}




method(get_descendants, SemsqlConn) <- function(
  x, term_id,
  predicates = "rdfs:subClassOf", include_self = FALSE
) {
  placeholders <- paste(rep("?", length(predicates)), collapse = ", ")

#  pred_list <- paste0("'", predicates, "'", collapse = ", ")
  query <- sprintf("SELECT
      ee.subject AS id,
      l.value AS label,
      ee.predicate
    FROM entailed_edge ee
    LEFT JOIN rdfs_label_statement l ON ee.subject = l.subject
    WHERE ee.object = ?
      AND ee.predicate IN (%s)
    ORDER BY l.value", placeholders)
  result <- dbGetQuery(x@con, query, param=c(list(term_id), as.list(predicates)))
  if (!include_self && nrow(result) > 0) {
    result <- result[result$id != term_id, , drop = FALSE]
  }
  result
}

# =============================================================================
# OWL RESTRICTION QUERY METHODS
# =============================================================================

method(get_restrictions, SemsqlConn) <- function(x, term_id) {
  query <- "SELECT
      svf.id AS restriction_id,
      svf.on_property AS property,
      pl.value AS property_label,
      svf.filler,
      fl.value AS filler_label
    FROM owl_some_values_from svf
    LEFT JOIN rdfs_label_statement pl ON svf.on_property = pl.subject
    LEFT JOIN rdfs_label_statement fl ON svf.filler = fl.subject
    WHERE svf.id IN (
      SELECT object FROM rdfs_subclass_of_statement WHERE subject = ?)"
  dbGetQuery(x@con, query, list(term_id))
}

method(find_by_restriction, SemsqlConn) <- function(
  x, property, filler,
  include_filler_descendants = FALSE) {
  if (include_filler_descendants) {
    query <- "SELECT DISTINCT
        sc.subject AS id,
        cl.value AS label
      FROM rdfs_subclass_of_statement sc
      JOIN owl_some_values_from svf ON sc.object = svf.id
      LEFT JOIN rdfs_label_statement cl ON sc.subject = cl.subject
      WHERE svf.on_property = ?
        AND svf.filler IN (
          SELECT subject FROM entailed_edge
          WHERE object = ? AND predicate = 'rdfs:subClassOf'
          UNION SELECT ?
        )
      ORDER BY cl.value"
      return(dbGetQuery(x@con, query, param=list(property, filler, filler)))
    } else {
    query <- "SELECT DISTINCT
        sc.subject AS id,
        cl.value AS label
      FROM rdfs_subclass_of_statement sc
      JOIN owl_some_values_from svf ON sc.object = svf.id
      LEFT JOIN rdfs_label_statement cl ON sc.subject = cl.subject
      WHERE svf.on_property = ?
        AND svf.filler = ?
      ORDER BY cl.value"
     return(dbGetQuery(x@con, query, param=list(property, filler)))
    }
 }

# =============================================================================
# COMPLEX QUERY METHODS
# =============================================================================

method(find_intersection, SemsqlConn) <- function(
  x, superclass_id,
  relation_property, related_to_id
) {
  query <- "WITH descendants AS (
      SELECT subject FROM entailed_edge
      WHERE object = ?  AND predicate = 'rdfs:subClassOf'
    ),
    related AS (
      SELECT sc.subject
      FROM rdfs_subclass_of_statement sc
      JOIN owl_some_values_from svf ON sc.object = svf.id
      WHERE svf.on_property = ?
        AND svf.filler IN (
          SELECT subject FROM entailed_edge
          WHERE object = ? AND predicate = 'rdfs:subClassOf'
          UNION SELECT ?
        )
    )
    SELECT
      d.subject AS id,
      l.value AS label
    FROM descendants d
    JOIN related r ON d.subject = r.subject
    LEFT JOIN rdfs_label_statement l ON d.subject = l.subject
    ORDER BY l.value"
  dbGetQuery(x@con, query, param=list(superclass_id, relation_property, related_to_id, related_to_id))
}

# =============================================================================
# STATISTICS METHODS
# =============================================================================

method(count_descendants, SemsqlConn) <- function(
  x, term_id,
  predicate = "rdfs:subClassOf"
) {
  query <- "SELECT COUNT(DISTINCT subject) AS n
    FROM entailed_edge
    WHERE object = ?
      AND predicate = ?"
  dbGetQuery(x@con, query, param=list(term_id, predicate))$n[1]
}

method(count_by_prefix, SemsqlConn) <- function(x) {
  query <- "
    SELECT
      SUBSTR(subject, 1, INSTR(subject, ':') - 1) AS prefix,
      COUNT(*) AS n
    FROM rdfs_label_statement
    WHERE subject LIKE '%:%'
    GROUP BY prefix
    ORDER BY n DESC
  "
  dbGetQuery(x@con, query)
}

# =============================================================================
# PRINT / REPORT METHODS
# =============================================================================


#' Show method for SemsqlConn.
#' Concise one-line summary displayed when a `SemsqlConn` object is
#' auto-printed at the R prompt.
#' @name print
#' @rdname print
#' @param x A `SemsqlConn` object.
#' @param ... not used
#' @return x, invisibly
#' @export
S7::method(print, SemsqlConn) <- function(x, ...) {
  if (!is_connected(x)) {
    stop("SemsqlConn object is disconnected; reconnect with semsql_connect()")
  }
  n_labels <- dbGetQuery(
    x@con,
    "SELECT COUNT(*) AS n FROM rdfs_label_statement"
  )$n
  cat(
    "<SemsqlConn>",
    " prefix:", x@ontology_prefix,
    " | labeled terms:", format(n_labels, big.mark = ","),
    "\n"
  )
  invisible(x)
}


#' @rdname report
#' @export
S7::method(report, SemsqlConn) <- function(object, ...) {
  cat("\n")
  cat(strrep("=", 60), "\n")
  cat("SemsqlConn Object\n")
  cat(strrep("=", 60), "\n\n")

  cat("Connection Details:\n")
  cat(strrep("-", 40), "\n")
  cat("  Database path:   ", object@db_path, "\n")
  cat("  Ontology prefix: ", object@ontology_prefix, "\n")

  connected <- is_connected(object)
  status_symbol <- if (connected) "\u2713" else "\u2717"
  status_text <- if (connected) "Connected" else "Disconnected"
  cat("  Status:          ", status_symbol, " ", status_text, "\n\n")

  if (!connected) {
    cat("(Connection closed - no further details available)\n\n")
    return(invisible(object))
  }

  cat("Database Statistics:\n")
  cat(strrep("-", 40), "\n")
  tryCatch(
    {
      n <- dbGetQuery(object@con, "SELECT COUNT(*) AS n FROM rdfs_label_statement")$n
      cat("  Labeled terms:   ", format(n, big.mark = ","), "\n")
    },
    error = function(e) cat("  Labeled terms:    (unavailable)\n")
  )
  tryCatch(
    {
      n <- dbGetQuery(object@con, "SELECT COUNT(*) AS n FROM edge")$n
      cat("  Direct edges:    ", format(n, big.mark = ","), "\n")
    },
    error = function(e) cat("  Direct edges:     (unavailable)\n")
  )
  tryCatch(
    {
      n <- dbGetQuery(object@con, "SELECT COUNT(*) AS n FROM entailed_edge")$n
      cat("  Entailed edges:  ", format(n, big.mark = ","), "\n")
    },
    error = function(e) cat("  Entailed edges:   (unavailable)\n")
  )
  tryCatch(
    {
      n <- dbGetQuery(
        object@con,
        "SELECT COUNT(*) AS n FROM has_text_definition_statement"
      )$n
      cat("  Definitions:     ", format(n, big.mark = ","), "\n")
    },
    error = function(e) cat("  Definitions:      (unavailable)\n")
  )

  cat("\n")
  cat("Terms by Prefix (top 5):\n")
  cat(strrep("-", 40), "\n")
  tryCatch(
    {
      top <- head(count_by_prefix(object), 5)
      cat(sprintf(
        "  %-16s %s",
        paste0(top$prefix,  ":"),
        formatC(top$n, big.mark = ",") 
      ), sep = "\n")
    },
    error = function(e) cat("  (unavailable)\n")
  )

  cat("\n")
  cat("Key Tables Available:\n")
  cat(strrep("-", 40), "\n")
  key_tables <- c(
    "rdfs_label_statement",
    "has_text_definition_statement",
    "edge",
    "entailed_edge",
    "rdfs_subclass_of_statement",
    "owl_some_values_from",
    "has_oio_synonym_statement"
  )
  all_tables <- list_tables(object)
  for (tbl in key_tables) {
    sym <- if (tbl %in% all_tables) "\u2713" else "\u2717"
    cat(" ", sym, "", tbl, "\n")
  }

  cat("\n")
  cat(strrep("=", 60), "\n")
  cat("Use methods like search_labels(), get_ancestors(), etc.\n")
  cat("Run ?SemsqlConn for documentation.\n")
  cat(strrep("=", 60), "\n\n")
  invisible(object)
}

# =============================================================================
# CONVENIENCE FUNCTIONS
# =============================================================================

#' Standard predicate CURIEs used in OBO ontologies
#'
#' @description
#' A named list of commonly used predicate CURIEs in OBO-format ontologies,
#' for use with [get_ancestors()], [get_descendants()],
#' and related functions.
#'
#' @format A named list with elements `subclass_of`, `part_of`,
#'   `has_part`, `develops_from`, `located_in`,
#'   `has_characteristic`.
#' @return a named list
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' # apoptotic process (GO:0006915) ancestors via is-a and part-of
#' anc <- get_ancestors(goref, "GO:0006915",
#'   predicates = c(PREDICATES$subclass_of, PREDICATES$part_of)
#' )
#' head(anc)
#' disconnect(goref)
#' @export
PREDICATES <- list(
  subclass_of = "rdfs:subClassOf",
  part_of = "BFO:0000050",
  has_part = "BFO:0000051",
  develops_from = "RO:0002202",
  located_in = "RO:0001025",
  has_characteristic = "RO:0000053"
)

#' Get ancestors traversing both is-a and part-of relationships
#'
#' @description
#' Convenience wrapper around [get_ancestors()] that follows both
#' `rdfs:subClassOf` and `BFO:0000050` (part-of) edges.
#'
#' @param conn A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param include_self logical(1) whether to include the term itself
#'   (default `FALSE`).
#' @return data.frame with columns `id`, `label`, `predicate`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_ancestors_partonomy(goref, "GO:0005739") # mitochondrion
#' disconnect(goref)
#' @export
get_ancestors_partonomy <- function(conn, term_id, include_self = FALSE) {
  get_ancestors(conn, term_id,
    predicates = c(PREDICATES$subclass_of, PREDICATES$part_of),
    include_self = include_self
  )
}

#' Get descendants traversing both is-a and has-part relationships
#'
#' @description
#' Convenience wrapper around [get_descendants()] that follows both
#' `rdfs:subClassOf` and `BFO:0000051` (has-part) edges.
#'
#' @param conn A `SemsqlConn` object.
#' @param term_id character(1) CURIE.
#' @param include_self logical(1) whether to include the term itself
#'   (default `FALSE`).
#' @return data.frame with columns `id`, `label`, `predicate`.
#' @examples
#' goref <- semsql_connect(ontology = "go")
#' get_descendants_partonomy(goref, "GO:0005634") # nucleus sub-components
#' disconnect(goref)
#' @export
get_descendants_partonomy <- function(conn, term_id, include_self = FALSE) {
  get_descendants(conn, term_id,
    predicates = c(PREDICATES$subclass_of, PREDICATES$has_part),
    include_self = include_self
  )
}

#' Execute code with an automatically managed SemsqlConn
#'
#' @description
#' Opens a connection, evaluates an expression with `conn` bound to the
#' open `SemsqlConn`, then closes the connection even if an error occurs.
#' Analogous to Python's context manager (`with` statement).
#'
#' @param db_path character(1) path to the SQLite database.
#' @param expr an expression to evaluate; `conn` is bound to the open
#'   `SemsqlConn` within this expression.
#' @return the value of `expr`.
#' @examples
#' \dontrun{
#' result <- with_connection("cl.db", {
#'   get_ancestors(conn, "CL:0000540")
#' })
#' }
#' @export
with_connection <- function(db_path, expr) {
  conn <- semsql_connect(db_path)
  on.exit(disconnect(conn, quiet = TRUE), add = TRUE)
  env <- new.env(parent = parent.frame())
  env$conn <- conn
  eval(substitute(expr), envir = env)
}

library(ontoProc2)

# Shared helper: pick the first AIO term with a label for use across tests
.get_test_term <- function(conn) {
  run_query(conn,
    "SELECT subject FROM rdfs_label_statement WHERE subject LIKE 'AIO:%' LIMIT 1"
  )$subject[1]
}

# ---------------------------------------------------------------------------
# search_labels
# ---------------------------------------------------------------------------

test_that("search_labels returns a data.frame with matching rows", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  result <- search_labels(conn, "a")
  expect_s3_class(result, "data.frame")
  expect_true(all(c("subject", "label") %in% names(result)))
  expect_true(nrow(result) > 0L)
})

test_that("search_labels respects the limit argument", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  result <- search_labels(conn, "a", limit = 3L)
  expect_lte(nrow(result), 3L)
})

test_that("search_labels returns 0-row data.frame for pattern with no match", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  result <- search_labels(conn, "zzzzz_no_match_xyzzy_99999")
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 0L)
})

# ---------------------------------------------------------------------------
# get_label
# ---------------------------------------------------------------------------

test_that("get_label returns data.frame for a known term", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_label(conn, term)
  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) >= 1L)
  expect_true("label" %in% names(result))
})

test_that("get_label returns 0-row data.frame for unknown term", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  result <- get_label(conn, "AIO:9999999999")
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 0L)
})

# ---------------------------------------------------------------------------
# get_definition
# ---------------------------------------------------------------------------

test_that("get_definition returns character(1) or NA for any term", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_definition(conn, term)
  expect_true(is.character(result) || is.na(result))
  expect_length(result, 1L)
})

test_that("get_definition returns NA for unknown term", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  result <- get_definition(conn, "AIO:9999999999")
  expect_true(is.na(result))
})

# ---------------------------------------------------------------------------
# get_synonyms
# ---------------------------------------------------------------------------

test_that("get_synonyms returns a data.frame for type='all'", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_synonyms(conn, term, type = "all")
  expect_s3_class(result, "data.frame")
  expect_true(all(c("subject", "predicate", "synonym") %in% names(result)))
})

test_that("get_synonyms accepts explicit synonym type", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  for (tp in c("exact", "broad", "narrow", "related")) {
    result <- get_synonyms(conn, term, type = tp)
    expect_s3_class(result, "data.frame")
  }
})

# ---------------------------------------------------------------------------
# get_term_info
# ---------------------------------------------------------------------------

test_that("get_term_info returns a list with expected elements", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  info <- get_term_info(conn, term)
  expect_type(info, "list")
  expect_true(all(c("id", "label", "definition", "synonyms",
                    "superclasses", "subclasses") %in% names(info)))
  expect_equal(info$id, term)
})

# ---------------------------------------------------------------------------
# get_direct_edges
# ---------------------------------------------------------------------------

test_that("get_direct_edges returns data.frame with expected columns", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  for (dir in c("outgoing", "incoming", "both")) {
    result <- get_direct_edges(conn, term, direction = dir)
    expect_s3_class(result, "data.frame")
    expect_true(all(c("subject", "predicate", "object") %in% names(result)))
  }
})

# ---------------------------------------------------------------------------
# get_direct_subclasses / get_direct_superclasses
# ---------------------------------------------------------------------------

test_that("get_direct_subclasses returns data.frame with id and label", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_direct_subclasses(conn, term)
  expect_s3_class(result, "data.frame")
  expect_true(all(c("id", "label") %in% names(result)))
})

test_that("get_direct_superclasses returns data.frame with id and label", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_direct_superclasses(conn, term)
  expect_s3_class(result, "data.frame")
  expect_true(all(c("id", "label") %in% names(result)))
})

# ---------------------------------------------------------------------------
# get_ancestors / get_descendants
# ---------------------------------------------------------------------------

test_that("get_ancestors returns data.frame with id, label, predicate", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_ancestors(conn, term)
  expect_s3_class(result, "data.frame")
  expect_true(all(c("id", "label", "predicate") %in% names(result)))
})

test_that("get_ancestors with include_self=TRUE includes the query term", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result_with <- get_ancestors(conn, term, include_self = TRUE)
  result_without <- get_ancestors(conn, term, include_self = FALSE)
  if (nrow(result_with) > 0) {
    expect_true(term %in% result_with$id)
  }
  if (nrow(result_without) > 0) {
    expect_false(term %in% result_without$id)
  }
})

test_that("get_descendants returns data.frame with id, label, predicate", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_descendants(conn, term)
  expect_s3_class(result, "data.frame")
  expect_true(all(c("id", "label", "predicate") %in% names(result)))
})

test_that("get_descendants with include_self=TRUE includes the query term", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result_with <- get_descendants(conn, term, include_self = TRUE)
  result_without <- get_descendants(conn, term, include_self = FALSE)
  if (nrow(result_with) > 0) {
    expect_true(term %in% result_with$id)
  }
  if (nrow(result_without) > 0) {
    expect_false(term %in% result_without$id)
  }
})

test_that("get_ancestors_partonomy returns data.frame", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_ancestors_partonomy(conn, term)
  expect_s3_class(result, "data.frame")
})

test_that("get_descendants_partonomy returns data.frame", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_descendants_partonomy(conn, term)
  expect_s3_class(result, "data.frame")
})

# ---------------------------------------------------------------------------
# count_descendants
# ---------------------------------------------------------------------------

test_that("count_descendants returns a non-negative integer", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  n <- count_descendants(conn, term)
  expect_type(n, "integer")
  expect_length(n, 1L)
  expect_gte(n, 0L)
})

# ---------------------------------------------------------------------------
# get_restrictions / find_by_restriction / find_intersection
# ---------------------------------------------------------------------------

test_that("get_restrictions returns a data.frame for any term", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- get_restrictions(conn, term)
  expect_s3_class(result, "data.frame")
  expected_cols <- c("restriction_id", "property", "property_label", "filler", "filler_label")
  expect_true(all(expected_cols %in% names(result)))
})

test_that("find_by_restriction returns a data.frame", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- find_by_restriction(conn, "BFO:0000050", term)
  expect_s3_class(result, "data.frame")
  expect_true(all(c("id", "label") %in% names(result)))
})

test_that("find_by_restriction with include_filler_descendants=TRUE returns data.frame", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- find_by_restriction(conn, "BFO:0000050", term,
                                include_filler_descendants = TRUE)
  expect_s3_class(result, "data.frame")
})

test_that("find_intersection returns a data.frame", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  term <- .get_test_term(conn)
  result <- find_intersection(conn, term, "BFO:0000050", term)
  expect_s3_class(result, "data.frame")
  expect_true(all(c("id", "label") %in% names(result)))
})

library(ontoProc2)

test_that("semsql_to_oi returns an ontology_index from aio connection", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  oi <- suppressWarnings(semsql_to_oi(conn@con))
  expect_true(inherits(oi, "ontology_index"))
})

test_that("semsql_to_oi result has name and parents elements", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  oi <- suppressWarnings(semsql_to_oi(conn@con))
  expect_true(all(c("name", "parents") %in% names(oi)))
})

test_that("semsql_to_oi result has at least one term", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  oi <- suppressWarnings(semsql_to_oi(conn@con))
  expect_true(length(oi$name) > 0L)
})

test_that("semsql_to_oi errors if required tables are absent", {
  tmp <- tempfile(fileext = ".db")
  on.exit(unlink(tmp))
  con <- DBI::dbConnect(RSQLite::SQLite(), tmp)
  DBI::dbExecute(con, "CREATE TABLE dummy (id TEXT)")
  expect_error(semsql_to_oi(con))
  DBI::dbDisconnect(con)
})

library(ontoProc2)

test_that("retriever behaves reasonably", {
  conn <- semsql_connect(ontology = "aio")
  expect_true(inherits(conn, "ontoProc2::SemsqlConn"))
  expect_true(get_prefix(conn) == "AIO")
  expect_true(nrow(count_by_prefix(conn)) > 0)
  disconnect(conn)
  expect_error({
    suppressWarnings(tryCatch(
      {
        retrieve_semsql_conn("xyzzyAA", quietly = TRUE)
      },
      error = function(e) {
        stop(paste0("Download failed: ", e$message))
      }
    ))
  })
})

library(ontoProc2)

# verify a simple connect task returns an entity of proper class
test_that("semsql_connect returns SemsqlConn for cached ontology", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  expect_true(inherits(conn, "ontoProc2::SemsqlConn"))
})

# verify that is_connected succeeds
test_that("is_connected returns TRUE for open connection", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  expect_true(is_connected(conn))
})

# verify that is_connected is sensitive to disconnection
test_that("disconnect closes connection and is_connected returns FALSE", {
  conn <- semsql_connect(ontology = "aio")
  expect_true(is_connected(conn))
  disconnect(conn, quiet = TRUE)
  expect_false(is_connected(conn))
})

# test the reconnect method
test_that("reconnect restores a disconnected connection", {
  conn <- semsql_connect(ontology = "aio")
  disconnect(conn, quiet = TRUE)
  expect_false(is_connected(conn))
  conn2 <- reconnect(conn)
  on.exit(disconnect(conn2, quiet = TRUE))
  expect_true(is_connected(conn2))
})

# reconnect() on an already-open connection returns a new SemsqlConn object
# (S7 value semantics mean the original is unchanged); both need disconnecting
test_that("reconnect on already-connected object returns a connected object", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  conn2 <- reconnect(conn)
  expect_true(is_connected(conn2))
  disconnect(conn2, quiet = TRUE)
})

# the prefix concept here is associated with the
# ontology provider's practice of using upper
# case for the CURIE prefixes (e.g., CL:0000555 would
# be a CURIE in cl.db.gz).
# Note: AIO stores its CURIEs as lowercase "aio:" internally;
# "AIO" comes from semsql_connect() calling toupper(ontology).
test_that("get_prefix returns the ontology prefix", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  expect_equal(get_prefix(conn), "AIO")
})

# check behavior of list_tables, and verify some assumptions
# on standard table names
test_that("list_tables returns a character vector of table names", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  tbls <- list_tables(conn)
  expect_type(tbls, "character")
  expect_true(length(tbls) > 0)
  expect_true("rdfs_label_statement" %in% tbls)
  expect_true("edge" %in% tbls)
  expect_true("entailed_edge" %in% tbls)
})

# note that 'PRAGMA table_info(my_table); will list columns, types, nullability, defaults'
# this is peculiar to SQLite
test_that("describe_table returns a data.frame with PRAGMA info", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  info <- describe_table(conn, "rdfs_label_statement")
  expect_s3_class(info, "data.frame")
  expect_true(nrow(info) > 0)
  expect_true("name" %in% names(info))
})

# verify that unknown tables can't be described
test_that("describe_table errors on unknown table", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  expect_error(describe_table(conn, "no_such_table_xyz"))
})

test_that("count_by_prefix returns a data.frame with prefix and n columns", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  df <- count_by_prefix(conn)
  expect_s3_class(df, "data.frame")
  expect_true(nrow(df) > 0)
  expect_true(all(c("prefix", "n") %in% names(df)))
  expect_true(any(tolower(df$prefix) == "aio"))
})

# run_query passes a caller-supplied SELECT statement to the read-only
# connection; the SQLITE_RO flag prevents any writes or DDL
test_that("run_query accepts a SELECT statement and returns a data.frame", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  result <- run_query(conn, "SELECT subject, value AS label FROM rdfs_label_statement LIMIT 3")
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 3L)
  expect_true(all(c("subject", "label") %in% names(result)))
})

test_that("report runs without error on connected object", {
  conn <- semsql_connect(ontology = "aio")
  on.exit(disconnect(conn, quiet = TRUE))
  expect_invisible(report(conn))
})

test_that("report runs without error on disconnected object", {
  conn <- semsql_connect(ontology = "aio")
  disconnect(conn, quiet = TRUE)
  expect_invisible(report(conn))
})

test_that("semsql_connect errors when db_path and ontology are both NULL", {
  expect_error(semsql_connect())
})

test_that("semsql_connect errors for non-existent db_path", {
  expect_error(semsql_connect(db_path = "/no/such/file.db"))
})

test_that("retrieve_semsql_conn errors for unknown ontology", {
  expect_error(
    suppressWarnings(tryCatch(
      retrieve_semsql_conn("xyzzy_notanontology_abc123", quietly = TRUE),
      error = function(e) stop(paste0("Download failed: ", e$message))
    ))
  )
})

test_that("with_connection evaluates expression and auto-disconnects", {
  conn <- semsql_connect(ontology = "aio")
  db_path <- conn@db_path
  disconnect(conn, quiet = TRUE)
  result <- with_connection(db_path, {
    run_query(conn, "SELECT COUNT(*) AS n FROM rdfs_label_statement")$n
  })
  expect_type(result, "integer")
  expect_true(result > 0L)
})

#' produce a table with cells exhibiting given proteins on plasma
#' membrane according to CL
#' @param curies a character vector in format "PR:nnnnnnnnn"
#' @return a data.frame with columns cl, celltype, pr, protein
#' @examples
#' # 2064: microsialin, 1874: KLRB1-like protein
#' cells_with_pmp(c("PR:000002064", "PR:000001874"))
#' @export
cells_with_pmp <- function(curies) {
  data("tag2cn", package = "ontoProc2")
  cl <- semsql_connect(ontology = "cl")
  pr <- semsql_connect(ontology = "pr")
  on.exit({
    disconnect(cl)
    disconnect(pr)
  })
# note: RO:0002104 = 'has plasma membrane part'
  cl4pmp <- dplyr::tbl(cl@con, "entailed_edge") |>
    dplyr::filter(object %in% curies, predicate == "RO:0002104") |>
    as.data.frame() |>
    dplyr::filter(startsWith(object, "PR:")) |>
    dplyr::select(subject, object) |>
    dplyr::mutate(cl = subject, prtag = object)
  mappr <- dplyr::tbl(pr@con, "rdfs_label_statement") |>
    dplyr::select(subject, value) |>
    as.data.frame() |>
    dplyr::filter(startsWith(subject, "PR:")) |>
    dplyr::mutate(prtag = subject)
  ans <- dplyr::left_join(cl4pmp, mappr, by = "prtag") |> dplyr::select(prtag, value, cl)
  ans$celltype <- tag2cn[ans$cl]
  ans
}

library(ontoProc2)

test_that("cells_with_pmp acquires records as expected", {
 x = cells_with_pmp(c("PR:000002064", "PR:000001874"))
 expect_true(inherits(x, "data.frame"))
 expect_true(nrow(x)>50)
})

test_that("get_present_pmp acquires records as expected", {
 x = get_present_pmp(c("CL:0000091", "CL:0000926"))
 expect_true(inherits(x, "data.frame"))
 expect_true(nrow(x)>10)
})

# not exported, script that creates saved data vectors
makeBundledMaps = function() {
 cl = semsql_connect(ontology="cl")
 thmap = dplyr::tbl(cl@con, "rdfs_label_statement") |> dplyr::filter(subject %like% "%CL:%") |> 
     select(subject, value) |> as.data.frame() 
 cn2tag = thmap$subject
 names(cn2tag) = thmap$value
# save(cn2tag, file="cn2tag.rda", compress="xz")
 tag2cn = thmap$value
 names(tag2cn) = thmap$subject
# save(tag2cn, file="tag2cn.rda", compress="xz")
 nn = semsql_connect(ontology="ncit")
 tbl(nn@con, "rdfs_label_statement") |> as.data.frame() -> newnc
 ncit_map = newnc$value
 names(ncit_map) = newnc$subject
 list(tag2cn=tag2cn, cn2tag=cn2tag, ncit_map=ncit_map)
}

#' produce an ontology_index instance from semantic sql sqlite connection
#' @importFrom ontologyIndex ontology_index
#' @param con DBI::dbConnect value for sqlite table
#' @return result of ontologyIndex::ontology_index evaluated for the labels and
#' parent-child relations in tables statements and edge of the semantic sql resource
#' @examples
#' \dontrun{
#' conn <- semsql_connect(ontology = "aio")
#' oi <- suppressWarnings(semsql_to_oi(conn@con))
#' names(oi)
#' }
#' @export
semsql_to_oi <- function(con) {
  # sqlite> select * from statements where predicate = 'rdfs:label' limit 40;
  alltabs <- DBI::dbListTables(con)
  stopifnot(all(c("edge", "statements") %in% alltabs))
  labdf <- dplyr::tbl(con, "statements") |>
    dplyr::filter(predicate == "rdfs:label") |>
    as.data.frame()
  edgdf <- dplyr::tbl(con, "edge") |>
    dplyr::filter(predicate == "rdfs:subClassOf") |>
    as.data.frame()
  nn <- split(labdf$value, labdf$subject) # value!!
  pl <- split(edgdf$object, edgdf$subject)
  okn <- intersect(names(nn), names(pl))
  pl <- pl[okn]
  nn <- nn[okn]
  ontologyIndex::ontology_index(name = nn, parents = pl)
}

#' produce INCAtools distribution URL
#' @param ontology short string that is the prefix to .db.gz in the bbop-sqlite collection
#' @return a string with URL for INCAtools resource
#' @examples
#' semsql_url("cl")
#' @export
semsql_url <- function(ontology = "efo") {
  sprintf("https://semanticsql.berkeleybop.io/%s.db.gz", ontology)
}

library(ontoProc2)

test_that("semsql_url formats URL correctly", {
  url <- semsql_url("cl")
  expect_type(url, "character")
  expect_length(url, 1L)
  expect_match(url, "^https://semanticsql.berkeleybop.io/cl\\.db\\.gz$")
})

test_that("semsql_url works for different ontologies", {
  expect_match(semsql_url("go"), "go\\.db\\.gz$")
  expect_match(semsql_url("efo"), "efo\\.db\\.gz$")
  expect_match(semsql_url("aio"), "aio\\.db\\.gz$")
})

test_that("PREDICATES is a named list with expected elements", {
  expect_type(PREDICATES, "list")
  expected <- c("subclass_of", "part_of", "has_part", "develops_from",
                "located_in", "has_characteristic")
  expect_true(all(expected %in% names(PREDICATES)))
})

test_that("PREDICATES contains correct CURIE values", {
  expect_equal(PREDICATES$subclass_of, "rdfs:subClassOf")
  expect_equal(PREDICATES$part_of, "BFO:0000050")
  expect_equal(PREDICATES$has_part, "BFO:0000051")
  expect_equal(PREDICATES$develops_from, "RO:0002202")
  expect_equal(PREDICATES$located_in, "RO:0001025")
  expect_equal(PREDICATES$has_characteristic, "RO:0000053")
})

#' produce a table with list of proteins from protein ontology
#' identified as present on cell membranes for input cell type CURIEs
#' @param curies a character vector in format "CL:nnnnnnn"
#' @return a data.frame with columns cl, celltype, pr, protein
#' @examples
#' get_present_pmp(c("CL:0000091", "CL:0000926"))
#' @export
get_present_pmp <- function(curies) {
  data("tag2cn", package = "ontoProc2")
  cl <- semsql_connect(ontology = "cl")
  pr <- semsql_connect(ontology = "pr")
  on.exit({
    disconnect(cl)
    disconnect(pr)
  })
  pmp4type <- dplyr::tbl(cl@con, "entailed_edge") |>
    dplyr::filter(subject %in% curies, predicate == "RO:0002104") |>
    as.data.frame() |>
    dplyr::filter(grepl("PR:", object)) |>
    dplyr::select(subject, object)
  ok <- pmp4type$object
  prmed <- dplyr::tbl(pr@con, "rdfs_label_statement") |>
    dplyr::filter(subject %in% ok) |>
    as.data.frame()
  lj <- dplyr::left_join(pmp4type |>
    dplyr::mutate(cl = subject, prtag = object), prmed |>
    dplyr::mutate(prtag = subject, prlab = value), by = "prtag") |> dplyr::select(cl, prtag, prlab)
  cn <- tag2cn[lj$cl]
  ans <- data.frame(cl = lj$cl, celltype = cn, pr = lj$prtag, protein = lj$prlab)
  ans |> dplyr::filter(pr != "PR:000000001")
}

#' return a SQLite connection (read only) to an INCAtools Semantic SQL ontology
#' @import BiocFileCache
#' @import RSQLite
#' @importFrom R.utils gunzip
#' @import DBI
#' @importFrom utils download.file
#' @param ontology character(1) short string prefixing .db.gz in the INCAtools collection
#' @param cache a BiocFileCache instance, defaulting to BiocFileCache::BiocFileCache()
#' @param cacheid character(1) or NULL; if non-null, the associated SQLite resource will be used from cache
#' @param \dots passed to download.file
#' @return an RSQLite DBI connection instance
#' @note When the cache is searched, the string given as 'ontology' will be prefixed with '^'.  This
#' helps avoid confusion between pcl.db and cl.db, for example.
#' @examples
#' # first time will involve a download and decompression
#' aionto <- retrieve_semsql_conn("aio")
#' head(DBI::dbListTables(aionto))
#' dplyr::tbl(aionto, "class_node") |> head()
#' @export
retrieve_semsql_conn <- function(ontology = "efo",
                                 cache = BiocFileCache::BiocFileCache(), cacheid = NULL, ...) {
  #
  # this function checks for cached version of ontology database
  # and if present, returns SQLite connection to it.  otherwise
  # it caches and then recurses
  #
  if (!is.null(cacheid)) {
    cached_path <- bfcinfo(cache, cacheid)$rpath
    return(dbConnect(SQLite(), cached_path, SQLITE_RO))
  }
  rname <- paste0(ontology, "_bbop_ontoproc2")
  bbop_info <- BiocFileCache::bfcquery(cache, rname)
  ind <- which(startsWith(bbop_info$rname, rname))
  if (length(ind) > 0) {
    if (length(ind) > 1) {
      message(
        sprintf("multiple cache entries found matching request %s", rname),
        "please be more specific, by supplying cache id as 'BFCnnn'.",
        sprintf("see %s in cache\n", paste(bbop_info$rid, collapse = ", "))
      )
      stop("cannot proceed with ambiguous ontology spec")
    }
    cached_path <- bbop_info[ind, "rpath"]$rpath # ind is length 1
    return(dbConnect(SQLite(), cached_path, SQLITE_RO))
  } else {
    addr <- semsql_url(ontology)
    zdbname <- basename(addr)
    dbname <- sub(".gz$", "", zdbname)
    td <- tempfile()
    dir.create(td)
    on.exit(unlink(td, recursive = TRUE))
    ztmploc <- file.path(td, zdbname)
    tmploc <- file.path(td, dbname)
    download.file(addr, file.path(td, zdbname), headers = c("User-Agent" = "ontoProc2"), ...)
    gunzip(ztmploc) # file now at tmploc
    addv <- BiocFileCache::bfcadd(cache,
      rname = rname,
      rtype = "local", fpath = tmploc, action = "copy"
    )
    id <- bfcquery(cache, basename(addv))$rid
    if (!isTRUE(startsWith(id, "BFC"))) stop("cache action failed") # avoid possibly infinite Recall
    Recall(ontology = ontology, cache = cache, cacheid = id)
  }
}

library(shiny)
library(ontoProc2)
learn_uberon = function(regexp="anatom", exclude="^GO:") {
 ubref = retrieve_semsql_conn("uberon")
 uoi = semsql_to_oi(ubref)
 DBI::dbDisconnect(ubref)
 nms = grep(regexp, uoi$name, value=TRUE)
 if (!is.null(exclude)) {
   drop = grep(exclude, names(nms))
   if (length(drop)>0) nms = nms[-drop]
   }
 tmp = names(nms)
 val = tmp
 names(val) = as.character(nms)
 ubapp(val, uoi)
}

ubapp = function(nms, uoi) {
 ui = fluidPage(
  sidebarLayout(
   sidebarPanel(
    helpText("ubapp"),
    helpText("pick a term"),
    radioButtons("nms", "candidates", nms)
   ),
   mainPanel(
    tabsetPanel(
     tabPanel("children",
      dataTableOutput("children")
      ),
     tabPanel("sibs",
      dataTableOutput("sibs")
      ),
     tabPanel("ancestors",
      dataTableOutput("ancestors")
      ),
     tabPanel("plot", plotOutput("plot")),
     tabPanel("about",
      helpText("about ubapp")
      )
     )
    )
   )
  )
  server = function(input, output) {
   sibs = function(x) {
     px = uoi$parents[[x]]
     unlist(uoi$children[unlist(px)])
   }
   output$sibs = renderDataTable({
    ss = sibs(input$nms)
    txt = as.character(unlist(uoi$name[ss]))
    ot = order(txt)
    data.frame(text=txt[ot], tag=ss[ot])
    })
   output$children = renderDataTable({
    chil = uoi$children[[input$nms]]
    validate(need(length(chil)>0, "no children, pick another"))
    txt = as.character(unlist(uoi$name[chil]))
    ot = order(txt)
    data.frame(text=txt[ot], tag=as.character(chil[ot]))
    })   
   output$ancestors = renderDataTable({
    anc = uoi$ancestors[[input$nms]]
    txt = as.character(unlist(uoi$name[anc]))
    data.frame(text=txt, tag=as.character(anc))
    })
   output$plot = renderPlot({
    onto_plot2(uoi, sibs(input$nms))
   })
  }
  runApp(list(ui=ui, server=server))
}
     
    

# this file was taken from ontoProc

#' obtain graphNEL from ontology_plot instance of ontologyPlot
#' @importFrom methods new as
#' @import graph
#' @import Rgraphviz
#' @importFrom ontologyPlot onto_plot
#' @param x instance of S3 class ontology_plot
#' @return instance of S4 graphNEL class
#' @examples
#' requireNamespace("Rgraphviz")
#' requireNamespace("graph")
#' clcon <- retrieve_semsql_conn("cl")
#' cl <- semsql_to_oi(clcon)
#' cl3k <- c(
#'   "CL:0000492", "CL:0001054", "CL:0000236", "CL:0000625",
#'   "CL:0000576", "CL:0000623", "CL:0000451", "CL:0000556"
#' )
#' p3k <- ontologyPlot::onto_plot(cl, cl3k)
#' gnel <- make_graphNEL_from_ontology_plot(p3k)
#' gnel <- improve_nodes(gnel, cl)
#' graph::graph.par(list(nodes = list(shape = "plaintext", cex = .8)))
#' gnel <- Rgraphviz::layoutGraph(gnel)
#' Rgraphviz::renderGraph(gnel)
#' @export
make_graphNEL_from_ontology_plot <- function(x) {
  ont_graph <- new(
    "graphAM",
    adjMat = x[["adjacency_matrix"]],
    edgemode = "directed"
  )
  as(ont_graph, "graphNEL")
}

#' inject linefeeds for node names for graph, with textual
#' annotation from ontology
#' @param g graphNEL instance
#' @param ont instance of ontology from ontologyIndex
#' @return graphNEL instance
#' @examples
#' requireNamespace("Rgraphviz")
#' requireNamespace("graph")
#' clcon <- retrieve_semsql_conn("cl")
#' cl <- semsql_to_oi(clcon)
#' cl3k <- c(
#'   "CL:0000492", "CL:0001054", "CL:0000236", "CL:0000625",
#'   "CL:0000576", "CL:0000623", "CL:0000451", "CL:0000556"
#' )
#' p3k <- ontologyPlot::onto_plot(cl, cl3k)
#' gnel <- make_graphNEL_from_ontology_plot(p3k)
#' head(graph::nodes(gnel)) # before improving
#' gnel <- improve_nodes(gnel, cl)
#' head(graph::nodes(gnel))
#' @export
improve_nodes <- function(g, ont) {
  nn <- paste0(sub(" ", "\n", ont$name[nodes(g)]), "\n", nodes(g))
  nodes(g) <- nn
  g
}

#' high-level use of graph/Rgraphviz for rendering ontology relations
#' @param ont instance of ontology from ontologyIndex
#' @param terms2use character vector
#' @param cex numeric(1) defaults to .8, supplied to Rgraphviz::graph.par
#' @param ... passed to onto_plot of ontologyPlot
#' @return graphNEL instance (invisibly)
#' @examples
#' clcon <- retrieve_semsql_conn("cl")
#' cl <- semsql_to_oi(clcon)
#' cl3k <- c(
#'   "CL:0000492", "CL:0001054", "CL:0000236", "CL:0000625",
#'   "CL:0000576", "CL:0000623", "CL:0000451", "CL:0000556"
#' )
#' onto_plot2(cl, cl3k)
#' @export
onto_plot2 <- function(ont, terms2use, cex = .8, ...) {
  pl <- ontologyPlot::onto_plot(ont, terms2use, ...)
  gnel <- make_graphNEL_from_ontology_plot(pl)
  gnel <- improve_nodes(gnel, ont)
  graph.par(list(nodes = list(shape = "plaintext", cex = cex)))
  gnel <- layoutGraph(gnel)
  renderGraph(gnel)
  invisible(gnel)
}

library(ontoProc2)

test_that("graphNEL functions succeed", {
 requireNamespace("Rgraphviz")
 requireNamespace("graph")
 clcon <- retrieve_semsql_conn("cl")
 expect_warning(cl <- semsql_to_oi(clcon)) # some parents not found, so warning thrown
 expect_true(inherits(cl, "ontology_index"))
 cl3k <- c(
   "CL:0000492", "CL:0001054", "CL:0000236", "CL:0000625",
   "CL:0000576", "CL:0000623", "CL:0000451", "CL:0000556"
 )
 p3k <- ontologyPlot::onto_plot(cl, cl3k)
 expect_true(inherits(p3k, "ontology_plot"))
 gnel <- make_graphNEL_from_ontology_plot(p3k)
 expect_true(inherits(gnel, "graphNEL"))
 head(graph::nodes(gnel)) # before improving
 gnel <- improve_nodes(gnel, cl)
 expect_true(length(grep("alpha-beta", graph::nodes(gnel)))>0)
})

#' a named vector with values rdfs labels in NCI thesaurus, and names the corresponding formal ontology tags
#' @note creation is detailed in unexported function makeBundledMaps
#' @importFrom utils data
#' @docType data
#' @format named character vector
#' @usage
#' data(ncit_map)
#' @examples
#' data("ncit_map", package = "ontoProc2")
#' head(ncit_map)
"ncit_map"

#' a named vector with mapping from CURIE to cell type phrase for CL.owl
#' @note creation is detailed in unexported function makeBundledMaps
#' @docType data
#' @format names character vector
#' @usage
#' data(tag2cn)
#' @examples
#' data("tag2cn", package = "ontoProc2")
#' tag2cn[c("CL:0000000", "CL:0000006")]
"tag2cn"

#' a named vector with mapping from cell type phrase to CURIE for CL.owl
#' @note creation is detailed in unexported function makeBundledMaps
#' @docType data
#' @format names character vector
#' @usage
#' data(cn2tag)
#' @examples
#' data("cn2tag", package = "ontoProc2")
#' cn2tag["Kupffer cell"]
"cn2tag"

#' retrieve and cache all filenames of Semantic SQL ontologies available, checking for updated
#' content relative to cache via ETag
#' @param url source address
#' @param bfc an instance of BiocFileCache
#' @param rname a string to use to identify the cached listing
#' @param timeout_connect passed to httr2::req_options connecttimeout parameter
#' @param timeout_total passed to httr2::req_options seconds parameter
#' @note There is no real etag discipline for the metadata, so full metadata
#' content is extracted on each call, digested, and compared to digest in cache.
#' Cache is updated when there is a discrepancy, except there is a guard against
#' rewriting cache with corrupted or zero-length data at endpoint.  Note this
#' code was negotiated at length with claude.ai Sonnet 4.6.
#' @return a character vector
#' @examples
#' gzs = bbop_sqlite_db_gz()
#' head(gzs)
#' length(gzs)
#' @export
bbop_sqlite_db_gz <- function(
    url = "https://semanticsql.berkeleybop.io/",
    bfc             = BiocFileCache::BiocFileCache(),
    rname           = "bbop_sqlite_listing",
    timeout_connect = 5,
    timeout_total   = 15) {

  # --- 1. Check cache --------------------------------------------------------
  hits       <- BiocFileCache::bfcquery(bfc, rname, field = "rname", exact = TRUE)
  have_cache <- nrow(hits) == 1L

  # --- 2. Try to fetch fresh listing -----------------------------------------
  xml_text     <- NULL
  current_hash <- NULL

  xml_text <- tryCatch({
    message("Fetching bucket listing...")
    txt <- httr2::request(url) |> httr2::req_user_agent(paste0("ontoProc2/", 
             utils::packageVersion("ontoProc2"))) |>
      httr2::req_options(connecttimeout = timeout_connect) |>
      httr2::req_timeout(seconds = timeout_total) |>
      httr2::req_perform() |>
      httr2::resp_body_string()
    current_hash <- digest::digest(txt, algo = "md5")
    txt
  }, error = function(e) {
    if (have_cache) {
      message("Could not reach ", url, "; using cached listing.")
      NULL
    } else {
      stop("Could not reach ", url, " and no cached listing exists. ",
           "Connect to the internet and retry.", call. = FALSE)
    }
  })

  # --- 3. Validate, compare hash, update cache only if safe ------------------
  if (!is.null(xml_text)) {

    # Parse and validate new content before touching anything
    new_keys <- tryCatch({
      doc <- xml2::read_xml(xml_text)
      ns  <- xml2::xml_ns(doc)
      tryCatch(
        xml2::xml_text(xml2::xml_find_all(doc, "//d1:Key", ns)),
        error = function(e)
          xml2::xml_text(xml2::xml_find_all(doc, "//*[local-name()='Key']"))
      )
    }, error = function(e) character(0))

    if (length(new_keys) == 0L) {
      warning("Freshly fetched listing contains no keys; ",
              if (have_cache) "retaining existing cache." else "nothing to cache.")
      if (!have_cache) return(character(0))
      # fall through to parse from existing cache below
      xml_text <- NULL

    } else {
      # Good content confirmed; now decide whether cache needs updating
      needs_update <- TRUE

      if (have_cache) {
        cached_path <- BiocFileCache::bfcrpath(bfc, rname)
        hash_path   <- paste0(cached_path, ".hash")

        if (file.exists(hash_path)) {
          cached_hash <- readLines(hash_path, warn = FALSE)
          if (length(cached_hash) == 1L && cached_hash == current_hash) {
            message("Content hash matches; using cached listing.")
            needs_update <- FALSE
          } else {
            message("Content changed (", length(new_keys), " keys found); ",
                    "updating cache.")
            file.remove(hash_path)
            BiocFileCache::bfcremove(bfc, hits$rid)
          }
        } else {
          message("No hash sidecar; updating cache.")
          BiocFileCache::bfcremove(bfc, hits$rid)
        }
      }

      if (needs_update) {
        cached_path <- BiocFileCache::bfcnew(bfc, rname, ext = ".xml")
        writeLines(xml_text, cached_path)
        writeLines(current_hash, paste0(cached_path, ".hash"))
        message("Cached with hash: ", current_hash)
      }
    }
  }

  # --- 4. Parse from cache and filter ----------------------------------------
  cached_path <- BiocFileCache::bfcrpath(bfc, rname)
  doc  <- xml2::read_xml(paste(readLines(cached_path, warn = FALSE), collapse = "\n"))
  ns   <- xml2::xml_ns(doc)

  keys <- tryCatch(
    xml2::xml_text(xml2::xml_find_all(doc, "//d1:Key", ns)),
    error = function(e)
      xml2::xml_text(xml2::xml_find_all(doc, "//*[local-name()='Key']"))
  )

  keys[endsWith(keys, "db.gz")]
}

---
title: "uberonpeek -- a look at UBERON ontology, etc., with ontoProc2"
author: "Vincent J. Carey, stvjc at channing.harvard.edu"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{uberonpeek -- a look at UBERON ontology, etc., with ontoProc2}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    highlight: pygments
    number_sections: yes
    theme: united
    toc: yes
---

# Introduction

The ontoProc2 package is designed to give convenient
access to the ontologies that are transformed to "semantic
SQL" in the INCAtools project.

We'll start by retrieving the current UBERON ontology and examining
some tables and "statements".

```{r setup,message=FALSE}
library(ontoProc2)
library(DBI)
library(dplyr)
ubss <- semsql_connect(ontology = "uberon")
report(ubss)
ubcon <- ubss@con
head(dbListTables(ubcon))
tbl(ubcon, "statements")
```

# Parent-child relations

CRAN's ontologyIndex package provides a familiar representation that
simplifies visualization.

```{r doconv, cache=FALSE}
uboi <- semsql_to_oi(ubcon)
uboi
uboi$name[10364:10370]
```

A sense of the variety of ontological cross-references
present can be given by tabling the tag prefixes.
```{r lktags}
prefs <- gsub(":.*", "", names(uboi$name))
table(prefs)
```

By using the ancestors component we can obtain
a view of is-a relations (presumably developed from
rdfs:subClassOf predicates).  We've chosen as
terminal tags the tags for heart, kidney, and cortex of kidney.
```{r doplot, fig.width=10}
onto_plot2(
  uboi,
  unlist(uboi$ancestors[c(
    "UBERON:0002189",
    "UBERON:0002113", "UBERON:0000948"
  )])
)
```

# Bridging to MONDO for disease terminology

With our knowledge of the tag for "heart", we can
enumerate formal terms for diseases affecting this organ.

```{r lkmondo}
mon = semsql_connect(ontology="mondo")
tbl(mon@con, "entailed_edge") |> 
   filter(object == "UBERON:0000948") |> 
   filter(subject %like% "MONDO%") |> 
   inner_join( tbl(mon@con, "rdfs_label_statement"), by="subject") |> 
   as.data.frame() |> select(subject, value) |> distinct() |> DT::datatable()
```


# Bridging to CL for cell type enumeration

```{r docross}
cl = semsql_connect(ontology="cl")
tbl(ubcon, "entailed_edge") |> 
   filter(object == "UBERON:0000948") |> 
   filter(subject %like% "CL:%") |> 
   inner_join( tbl(cl@con, "rdfs_label_statement"), by="subject", copy="temp-table") |> 
   as.data.frame() |> select(subject, value) |> distinct() |> DT::datatable()
```

Exercise: create a map from cardiac diseases to associated cardiac cell types.

# Session information

```{r lksess}
sessionInfo()
```

---
title: "ontoProc2 -- leveraging semantic SQL for ontology analysis in Bioconductor"
author: "Vincent J. Carey, stvjc at channing.harvard.edu"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{ontoProc2 -- leveraging INCAtools semantic SQL for ontology analysis in Bioconductor}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    highlight: pygments
    number_sections: yes
    theme: united
    toc: yes
---

# Introduction

The ontoProc2 package has two aims:

- to give convenient
access to the ontologies that are transformed to "semantic
SQL" in the [INCAtools semantic SQL project](https://github.com/INCATools/semantic-sql);
- to simplify operations that have been available in [ontoProc](https://git.bioconductor.org/packages/ontoProc), which will be deprecated in 2027.

This package is a "second generation" approach to ontology management
and processing for Bioconductor.  The ontoProc package was introduced in
2018 and had a number of experimental
approaches to interface design and content caching.  This package
streamlines the identification and management of diverse ontologies
by specifically leveraging semantic SQL concepts.

Specifically, the call `ontoProc::getOnto('cellOnto')` would produce
an `ontologyIndex` instance.  To work with Cell Ontology with ontoProc2,
use `semsql_connect(ontology="cl")`.  The abbreviated notation for ontologies follows
that of the Open Biological and Biomedical Ontologies ([OBO](https://github.com/OBOFoundry/)) Foundry.

# Installation

Prior to the release of Bioconductor 3.24, use `BiocManager::install("vjcitn/ontoProc2")`.

For Bioconductor 3.24 and subsequent versions, use `BiocManager::install("ontoProc2")`.

# Acquiring ontologies

## Make a connection

The best way to work with an ontology in this system is
to use `semsql_connect`.  The `ontology` argument will
be a short string that the INCAtools project uses as part
of the filename for the ontology.  For Gene Ontology, the
string is "go".

```{r doini, message=FALSE}
library(ontoProc2)
goss <- semsql_connect(ontology = "go")
goss
```
## Make a report

The `report` method provides details.
```{r lkrep1}
report(goss)
```

## Probe the back end

The back end is SQLite.  We can enumerate the
tables available:
```{r lktbs,message=FALSE}
library(dplyr)
library(DBI)
allt <- dbListTables(goss@con)
length(allt)
head(allt)
```

## Use tidy methods

Individual tables are readily accessible.

```{r lkti,message=FALSE}
library(DT)
tbl(goss@con, "statements")
tbl(goss@con, "statements") |>
  head(20) |>
  as.data.frame() |>
  datatable()
```

To investigate the ontology, searching through RDF labels is a natural approach.

```{r dosrc}
search_labels(goss, "apoptosis") |>
  head() |>
  datatable()
```

Additional filtering could be useful here to focus on GO terms.  The `_riog...`
labels have special roles in RDF inference, and this will be addressed in 
vignettes to be added in the future.

Let's improve the query:
```{r dosrc2}
search_labels(goss, "apoptosis") |>
  filter(grepl("^GO:", subject)) |>
  head() |>
  datatable()
```
Clearly it will be valuable to filter away obsolete terms.  We will investigate
the use of edge tables to accomplish this in a future vignette.


# Transformation to ontology_index instances

The [ontologyX suite](https://academic.oup.com/bioinformatics/article/33/7/1104/2843897) of
Daniel Greene and colleagues provides very convenient ontology handling functions.
We can transform the SQLite data to this format.  We'll illustrate with cell ontology.

```{r lkoi, cache=TRUE}
clss <- semsql_connect(ontology = "cl")
cloi <- semsql_to_oi(clss@con)
cloi
```

A convenience function assists with visualizations:

```{r dopl}
onto_plot2(cloi, c("CL:0000624", "CL:0000492", "CL:0000793", "CL:0000803"))
```

# Background

The S7 class design in this package was initiated by a request to Anthropic Claude
to use S7 in establishing code that mirrors the tasks accomplished in the
[INCAtools jupyter notebook](https://github.com/INCATools/semantic-sql/blob/main/notebooks/SemanticSQL-Tutorial.ipynb).

## Searching in label text

The code of `search_labels` is:
```{r lksrch}
library(S7)
method(search_labels, SemsqlConn)
```

## Exploring concept properties with 'edge tables'

The INCAtools notebook discusses the fact that `rdfs_label_statement` is a SQLite table "view".

The notebook indicates that a SPARQL query on an RDF store for the following computation
would be "quite hard".  We want to find all the "edges" leading from "enteric neuron", which
would constitute the set of subject-predicate-object statements about this cell type with "enteric
neuron" as subject.

In this code we use the concept of a "CURIE" (Compact Uniform Resource Identifier):
a fixed length numerical identifier with
a prefix indicating the source ontology in which the ontologic concept is based.

```{r doentr}
if (!is_connected(clss)) clss <- reconnect(clss)
entcurie <- search_labels(clss, "enteric neuron") |>
  filter(grepl("^CL", subject)) |>
  dplyr::select(subject) |>
  unlist()
entcurie
get_direct_edges(clss, entcurie)
```

Here the underlying code is performing a join:
```{r lkmeth}
method(get_direct_edges, SemsqlConn)
```

## Generalizing a concept: Ancestors

The notebook mentions that the "entailed edges" table includes
all statements that can be inferred from the application of
base axioms of the ontology.

```{r lkent}
get_ancestors(clss, entcurie)
```

## Working with multiple ontologies

The INCAtools notebook includes an example of finding all neurons
that are part of the forebrain.  This involves identifying
CURIEs for relations and anatomical structures, thus working
with the relational ontology (RO) and UBERON.

```{r getmore}
ub <- semsql_connect(ontology = "uberon")
ro <- semsql_connect(ontology = "ro")
```

First question: What's the CURIE for "forebrain" in UBERON?
```{r lkub}
fbcur <- search_labels(ub, "forebrain", limit = 1000) |>
  filter(label == "forebrain") |>
  select(subject) |>
  unlist()
fbcur
```
Second question: What's the CURIE for "has soma location" in RO?
```{r lkro}
loccur <- search_labels(ro, "has soma location") |>
  select(subject) |>
  unlist()
loccur
```
What's the CURIE for "neuron"?
```{r lkcurn}
ncur <- search_labels(clss, "neuron", limit = 1000) |>
  filter(label == "neuron") |>
  select(subject) |>
  unlist()
ncur
```

Now we use three steps to obtain the solution.

First, enumerate all cell types that are located in forebrain.
```{r infb}
clinfb <- tbl(clss@con, "entailed_edge") |>
  filter(predicate == loccur, object == fbcur) |>
  select(subject) |>
  collect() |>
  unlist()
length(clinfb)
```

Second, filter these to those identified as 'subclassOf' "neuron".
```{r isne}
clisneur <- tbl(clss@con, "entailed_edge") |>
  filter(predicate == "rdfs:subClassOf", object == ncur) |>
  filter(subject %in% clinfb) |>
  select(subject) |>
  collect() |>
  unlist()
length(clisneur)
```

Finally, get the labels.
```{r doint}
tbl(clss@con, "rdfs_label_statement") |>
  filter(subject %in% clisneur) |>
  select(subject, value) |>
  collect() |>
  DT::datatable()
```

# Session information

```{r lksess}
sessionInfo()
```

---
title: "celltoprotein -- connecting Cell and Protein Ontologies"
author: "Vincent J. Carey, stvjc at channing.harvard.edu"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{celltoprotein -- connecting Cell and Protein Ontologies}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    highlight: pygments
    number_sections: yes
    theme: united
    toc: yes
---

# Introduction

In a pair of papers from the Ventner Institute, 
[Bakken et al.](https://link.springer.com/article/10.1186/s12859-017-1977-1) and
[Aevermann et al.](https://academic.oup.com/hmg/article/27/R1/R40/4953379) discuss
ontological implications of single-cell transcriptomics.  A process of cell type
definition via "necessary and
sufficient marker gene" enumeration is introduced.

In this vignette we indicate how Cell Ontology, Relational Ontology,
and Protein Ontology can be connected to assess formal relationships
between declared cell types and plasma membrane features that can
play a role in cell type definition.

# Given a cell type, what proteins are noted as parts of its plasma membrane?

Connect to the relational ontology and search for CURIEs related
to "plasma membrane".
```{r lkon1,message=FALSE}
library(ontoProc2)
library(DT)
ro <- semsql_connect(ontology = "ro")
search_labels(ro, "plasma membrane")
```

We have a helper resource for finding exact Cell Ontology names
of cell types.

```{r lkct1}
data("tag2cn", package = "ontoProc2")
cd8reg <- grep("CD8-positive.*regulatory", tag2cn, value = TRUE)
cd8reg
```

Now with these cell type identifiers, we can search for the
proteins identified as "part of plasma membrane".
We need to use the CURIEs for precision.  THIS IS BLOCKED
UNTIL WE HAVE A SUBSET OF PR DATA TO ILLUSTRATE AS THE
PR DOWNLOADS ARE TOO SLOW.
```{r lkct2, message=FALSE,eval=FALSE}
prtab <- get_present_pmp(names(cd8reg))
datatable(prtab)
```

# Given a protein, what cell types are asserted to possess it as a membrane part?

We pick two proteins and look for associated cell types.  BLOCKED AS ABOVE.

```{r lkpr1, message=FALSE,eval=FALSE}
prs <- c("PR:000001094", "PR:000001380")
clk <- try(cells_with_pmp(prs))
if (inherits(clk, "try-error")) message("it is necessary to allow a large download of Protein Ontology for this chunk to run") else datatable(clk)
```

# Some details

The "entailed edge" table of the Semantic SQL representation of
Cell Ontology includes all assertions that are derivable from base
axioms of the ontology.  

```{r lkentedg, message=FALSE}
cl <- semsql_connect(ontology = "cl")
cl
library(dplyr)
tbl(cl@con, "entailed_edge")
tbl(cl@con, "entailed_edge") |> count()
```

We can look for statements that have "RO:0002104" as predicate:
```{r domo}
tbl(cl@con, "entailed_edge") |>
  filter(predicate == "RO:0002104") |>
  as.data.frame() |>
  filter(grepl("PR:", object)) |>
  arrange(subject) |>
  datatable()
```

Disconnect databases.

```{r dodisc}
disconnect(cl)
disconnect(ro)
```



# Session information

```{r lksess}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{get_direct_subclasses}
\alias{get_direct_subclasses}
\title{Get direct subclasses of a term}
\usage{
get_direct_subclasses(x, term_id, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{term_id}{character(1) CURIE.}

\item{...}{not used}
}
\value{
data.frame with columns \code{id} and \code{label}, ordered by
label.
}
\description{
Get direct subclasses of a term
}
\examples{
goref <- semsql_connect(ontology = "go")
# direct children of "apoptotic process" (GO:0006915)
get_direct_subclasses(goref, "GO:0006915")
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{with_connection}
\alias{with_connection}
\title{Execute code with an automatically managed SemsqlConn}
\usage{
with_connection(db_path, expr)
}
\arguments{
\item{db_path}{character(1) path to the SQLite database.}

\item{expr}{an expression to evaluate; \code{conn} is bound to the open
\code{SemsqlConn} within this expression.}
}
\value{
the value of \code{expr}.
}
\description{
Opens a connection, evaluates an expression with \code{conn} bound to the
open \code{SemsqlConn}, then closes the connection even if an error occurs.
Analogous to Python's context manager (\code{with} statement).
}
\examples{
\dontrun{
result <- with_connection("cl.db", {
  get_ancestors(conn, "CL:0000540")
})
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphNEL.R
\name{improve_nodes}
\alias{improve_nodes}
\title{inject linefeeds for node names for graph, with textual
annotation from ontology}
\usage{
improve_nodes(g, ont)
}
\arguments{
\item{g}{graphNEL instance}

\item{ont}{instance of ontology from ontologyIndex}
}
\value{
graphNEL instance
}
\description{
inject linefeeds for node names for graph, with textual
annotation from ontology
}
\examples{
requireNamespace("Rgraphviz")
requireNamespace("graph")
clcon <- retrieve_semsql_conn("cl")
cl <- semsql_to_oi(clcon)
cl3k <- c(
  "CL:0000492", "CL:0001054", "CL:0000236", "CL:0000625",
  "CL:0000576", "CL:0000623", "CL:0000451", "CL:0000556"
)
p3k <- ontologyPlot::onto_plot(cl, cl3k)
gnel <- make_graphNEL_from_ontology_plot(p3k)
head(graph::nodes(gnel)) # before improving
gnel <- improve_nodes(gnel, cl)
head(graph::nodes(gnel))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{count_by_prefix}
\alias{count_by_prefix}
\title{Count labeled terms grouped by CURIE prefix}
\usage{
count_by_prefix(x, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{...}{not used.}
}
\value{
data.frame with columns \code{prefix} and \code{n}, ordered by
\code{n} descending.
}
\description{
Count labeled terms grouped by CURIE prefix
}
\examples{
goref <- semsql_connect(ontology = "go")
count_by_prefix(goref)
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{get_restrictions}
\alias{get_restrictions}
\title{Get OWL someValuesFrom restrictions for a term}
\usage{
get_restrictions(x, term_id, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{term_id}{character(1) CURIE.}

\item{...}{not used}
}
\value{
data.frame with columns \code{restriction_id}, \code{property},
\code{property_label}, \code{filler}, \code{filler_label}.
}
\description{
Get OWL someValuesFrom restrictions for a term
}
\examples{
goref <- semsql_connect(ontology = "go")
# mitochondrion (GO:0005739) has part-of restrictions to cell
get_restrictions(goref, "GO:0005739")
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{get_descendants_partonomy}
\alias{get_descendants_partonomy}
\title{Get descendants traversing both is-a and has-part relationships}
\usage{
get_descendants_partonomy(conn, term_id, include_self = FALSE)
}
\arguments{
\item{conn}{A \code{SemsqlConn} object.}

\item{term_id}{character(1) CURIE.}

\item{include_self}{logical(1) whether to include the term itself
(default \code{FALSE}).}
}
\value{
data.frame with columns \code{id}, \code{label}, \code{predicate}.
}
\description{
Convenience wrapper around \code{\link[=get_descendants]{get_descendants()}} that follows both
\code{rdfs:subClassOf} and \code{BFO:0000051} (has-part) edges.
}
\examples{
goref <- semsql_connect(ontology = "go")
get_descendants_partonomy(goref, "GO:0005634") # nucleus sub-components
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphNEL.R
\name{onto_plot2}
\alias{onto_plot2}
\title{high-level use of graph/Rgraphviz for rendering ontology relations}
\usage{
onto_plot2(ont, terms2use, cex = 0.8, ...)
}
\arguments{
\item{ont}{instance of ontology from ontologyIndex}

\item{terms2use}{character vector}

\item{cex}{numeric(1) defaults to .8, supplied to Rgraphviz::graph.par}

\item{...}{passed to onto_plot of ontologyPlot}
}
\value{
graphNEL instance (invisibly)
}
\description{
high-level use of graph/Rgraphviz for rendering ontology relations
}
\examples{
clcon <- retrieve_semsql_conn("cl")
cl <- semsql_to_oi(clcon)
cl3k <- c(
  "CL:0000492", "CL:0001054", "CL:0000236", "CL:0000625",
  "CL:0000576", "CL:0000623", "CL:0000451", "CL:0000556"
)
onto_plot2(cl, cl3k)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_present_pmp.R
\name{get_present_pmp}
\alias{get_present_pmp}
\title{produce a table with list of proteins from protein ontology
identified as present on cell membranes for input cell type CURIEs}
\usage{
get_present_pmp(curies)
}
\arguments{
\item{curies}{a character vector in format "CL:nnnnnnn"}
}
\value{
a data.frame with columns cl, celltype, pr, protein
}
\description{
produce a table with list of proteins from protein ontology
identified as present on cell membranes for input cell type CURIEs
}
\examples{
get_present_pmp(c("CL:0000091", "CL:0000926"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/semsql_to_oi.R
\name{semsql_to_oi}
\alias{semsql_to_oi}
\title{produce an ontology_index instance from semantic sql sqlite connection}
\usage{
semsql_to_oi(con)
}
\arguments{
\item{con}{DBI::dbConnect value for sqlite table}
}
\value{
result of ontologyIndex::ontology_index evaluated for the labels and
parent-child relations in tables statements and edge of the semantic sql resource
}
\description{
produce an ontology_index instance from semantic sql sqlite connection
}
\examples{
\dontrun{
conn <- semsql_connect(ontology = "aio")
oi <- suppressWarnings(semsql_to_oi(conn@con))
names(oi)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontoProc2_package.R
\docType{package}
\name{ontoProc2-package}
\alias{ontoProc2}
\alias{ontoProc2-package}
\title{ontoProc2: Ontology Facilities Based on INCAtools Semantic SQL}
\description{
This package provides ontology facilities based on INCAtools Semantic SQL. Tooling is provided to retrieve and cache SQLite databases representing curated ontologies. ontologyIndex ontology_index instances can be produced. Anthropic's Claude was used in the development of S7 classes and methods and in the production of documentation.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/vjcitn/ontoProc2}
  \item Report bugs at \url{https://github.com/vjcitn/ontoProc2/issues}
}

}
\author{
\strong{Maintainer}: Vincent Carey \email{stvjc@channing.harvard.edu} (\href{https://orcid.org/0000-0003-4046-0063}{ORCID})

Authors:
\itemize{
  \item Vincent Carey \email{stvjc@channing.harvard.edu} (\href{https://orcid.org/0000-0003-4046-0063}{ORCID})
}

Other contributors:
\itemize{
  \item Anthropic-Claude Sonnet4.6 [engineer]
  \item NIH NHGRI 2U24HG004059-17 [funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{list_tables}
\alias{list_tables}
\title{List tables in a SemsqlConn database}
\usage{
list_tables(x, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{...}{not used}
}
\value{
character vector of table names.
}
\description{
List tables in a SemsqlConn database
}
\examples{
goref <- semsql_connect(ontology = "go")
list_tables(goref)
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{get_prefix}
\alias{get_prefix}
\title{Retrieve the ontology prefix from a SemsqlConn}
\usage{
get_prefix(x, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{...}{not used}
}
\value{
character(1) the primary ontology prefix (e.g. \code{"GO"}).
}
\description{
Retrieve the ontology prefix from a SemsqlConn
}
\examples{
goref <- semsql_connect(ontology = "go")
get_prefix(goref)
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/semsql_url.R
\name{semsql_url}
\alias{semsql_url}
\title{produce INCAtools distribution URL}
\usage{
semsql_url(ontology = "efo")
}
\arguments{
\item{ontology}{short string that is the prefix to .db.gz in the bbop-sqlite collection}
}
\value{
a string with URL for INCAtools resource
}
\description{
produce INCAtools distribution URL
}
\examples{
semsql_url("cl")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retrieve_semsql.R
\name{retrieve_semsql_conn}
\alias{retrieve_semsql_conn}
\title{return a SQLite connection (read only) to an INCAtools Semantic SQL ontology}
\usage{
retrieve_semsql_conn(
  ontology = "efo",
  cache = BiocFileCache::BiocFileCache(),
  cacheid = NULL,
  ...
)
}
\arguments{
\item{ontology}{character(1) short string prefixing .db.gz in the INCAtools collection}

\item{cache}{a BiocFileCache instance, defaulting to BiocFileCache::BiocFileCache()}

\item{cacheid}{character(1) or NULL; if non-null, the associated SQLite resource will be used from cache}

\item{\dots}{passed to download.file}
}
\value{
an RSQLite DBI connection instance
}
\description{
return a SQLite connection (read only) to an INCAtools Semantic SQL ontology
}
\note{
When the cache is searched, the string given as 'ontology' will be prefixed with '^'.  This
helps avoid confusion between pcl.db and cl.db, for example.
}
\examples{
# first time will involve a download and decompression
aionto <- retrieve_semsql_conn("aio")
head(DBI::dbListTables(aionto))
dplyr::tbl(aionto, "class_node") |> head()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ncit_map}
\alias{ncit_map}
\title{a named vector with values rdfs labels in NCI thesaurus, and names the corresponding formal ontology tags}
\format{
named character vector
}
\usage{
data(ncit_map)
}
\description{
a named vector with values rdfs labels in NCI thesaurus, and names the corresponding formal ontology tags
}
\note{
creation is detailed in unexported function makeBundledMaps
}
\examples{
data("ncit_map", package = "ontoProc2")
head(ncit_map)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{semsql_connect}
\alias{semsql_connect}
\title{Create a SemsqlConn connection}
\usage{
semsql_connect(
  db_path = NULL,
  ontology_prefix = NULL,
  ontology = NULL,
  cache = BiocFileCache::BiocFileCache(),
  validate = TRUE,
  ...
)
}
\arguments{
\item{db_path}{character(1) or NULL. Path to an existing SQLite database
file. Either \code{db_path} or \code{ontology} must be supplied.}

\item{ontology_prefix}{character(1) or NULL. Primary CURIE prefix for the
ontology (e.g. \code{"CL"}). If NULL and \code{ontology} is supplied,
defaults to \code{toupper(ontology)}; otherwise auto-detected from the
database.}

\item{ontology}{character(1) or NULL. Short name of an INCAtools ontology
(e.g. \code{"cl"}, \code{"go"}). If supplied,
\code{\link[=retrieve_semsql_conn]{retrieve_semsql_conn()}} is called to locate or download the
cached database.}

\item{cache}{a \code{BiocFileCache} instance used when \code{ontology} is
supplied. Defaults to \code{BiocFileCache::BiocFileCache()}.}

\item{validate}{logical(1) if TRUE (the default value) the ontology code
is checked against available Semantic SQL resources at INCAtools.  Set to
FALSE if using off line.}

\item{...}{passed to \code{\link[=retrieve_semsql_conn]{retrieve_semsql_conn()}} and ultimately
to \code{\link[utils:download.file]{utils::download.file()}}.}
}
\value{
A \code{\link[=SemsqlConn]{SemsqlConn()}} object.
}
\description{
Opens a connection to a SemanticSQL SQLite database, either by supplying
a direct file path or by referencing a short ontology name that is
retrieved and cached via \code{BiocFileCache}.
}
\note{
The connection has flag \code{SQLITE_RO} for read-only access.  There will
be an attempt to validate the \code{ontology} tag that is supplied, against all
available Semantic SQL resources available at INCAtools bucket.  Function fails
if a match cannot be made, which in general requires network access.
}
\examples{
# by ontology short name (downloads if not cached)
goref <- semsql_connect(ontology = "go")
goref
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphNEL.R
\name{make_graphNEL_from_ontology_plot}
\alias{make_graphNEL_from_ontology_plot}
\title{obtain graphNEL from ontology_plot instance of ontologyPlot}
\usage{
make_graphNEL_from_ontology_plot(x)
}
\arguments{
\item{x}{instance of S3 class ontology_plot}
}
\value{
instance of S4 graphNEL class
}
\description{
obtain graphNEL from ontology_plot instance of ontologyPlot
}
\examples{
requireNamespace("Rgraphviz")
requireNamespace("graph")
clcon <- retrieve_semsql_conn("cl")
cl <- semsql_to_oi(clcon)
cl3k <- c(
  "CL:0000492", "CL:0001054", "CL:0000236", "CL:0000625",
  "CL:0000576", "CL:0000623", "CL:0000451", "CL:0000556"
)
p3k <- ontologyPlot::onto_plot(cl, cl3k)
gnel <- make_graphNEL_from_ontology_plot(p3k)
gnel <- improve_nodes(gnel, cl)
graph::graph.par(list(nodes = list(shape = "plaintext", cex = .8)))
gnel <- Rgraphviz::layoutGraph(gnel)
Rgraphviz::renderGraph(gnel)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{run_query}
\alias{run_query}
\title{Run an arbitrary SQL query against a SemsqlConn database}
\usage{
run_query(x, sql, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{sql}{character(1) SQL query string.}

\item{...}{not used}
}
\value{
data.frame with query results.
}
\description{
Run an arbitrary SQL query against a SemsqlConn database
}
\examples{
goref <- semsql_connect(ontology = "go")
run_query(
  goref,
  "SELECT subject, value AS label FROM rdfs_label_statement LIMIT 5"
)
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{PREDICATES}
\alias{PREDICATES}
\title{Standard predicate CURIEs used in OBO ontologies}
\format{
A named list with elements \code{subclass_of}, \code{part_of},
\code{has_part}, \code{develops_from}, \code{located_in},
\code{has_characteristic}.
}
\usage{
PREDICATES
}
\value{
a named list
}
\description{
A named list of commonly used predicate CURIEs in OBO-format ontologies,
for use with \code{\link[=get_ancestors]{get_ancestors()}}, \code{\link[=get_descendants]{get_descendants()}},
and related functions.
}
\examples{
goref <- semsql_connect(ontology = "go")
# apoptotic process (GO:0006915) ancestors via is-a and part-of
anc <- get_ancestors(goref, "GO:0006915",
  predicates = c(PREDICATES$subclass_of, PREDICATES$part_of)
)
head(anc)
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tag2cn}
\alias{tag2cn}
\title{a named vector with mapping from CURIE to cell type phrase for CL.owl}
\format{
names character vector
}
\usage{
data(tag2cn)
}
\description{
a named vector with mapping from CURIE to cell type phrase for CL.owl
}
\note{
creation is detailed in unexported function makeBundledMaps
}
\examples{
data("tag2cn", package = "ontoProc2")
tag2cn[c("CL:0000000", "CL:0000006")]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{get_direct_edges}
\alias{get_direct_edges}
\title{Get direct edges in the ontology graph for a term}
\usage{
get_direct_edges(
  x,
  term_id,
  direction = c("outgoing", "incoming", "both"),
  ...
)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{term_id}{character(1) CURIE.}

\item{direction}{character(1) one of \code{"outgoing"}, \code{"incoming"},
\code{"both"}.}

\item{...}{not used}
}
\value{
data.frame with columns \code{subject}, \code{subject_label},
\code{predicate}, \code{predicate_label}, \code{object},
\code{object_label}.
}
\description{
Get direct edges in the ontology graph for a term
}
\examples{
goref <- semsql_connect(ontology = "go")
get_direct_edges(goref, "GO:0006915")
get_direct_edges(goref, "GO:0006915", direction = "both")
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cn2tag}
\alias{cn2tag}
\title{a named vector with mapping from cell type phrase to CURIE for CL.owl}
\format{
names character vector
}
\usage{
data(cn2tag)
}
\description{
a named vector with mapping from cell type phrase to CURIE for CL.owl
}
\note{
creation is detailed in unexported function makeBundledMaps
}
\examples{
data("cn2tag", package = "ontoProc2")
cn2tag["Kupffer cell"]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{find_by_restriction}
\alias{find_by_restriction}
\title{Find terms that have a given OWL someValuesFrom restriction}
\usage{
find_by_restriction(
  x,
  property,
  filler,
  include_filler_descendants = FALSE,
  ...
)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{property}{character(1) property CURIE
(e.g. \code{"BFO:0000050"} for part-of).}

\item{filler}{character(1) filler class CURIE.}

\item{include_filler_descendants}{logical(1) if TRUE also match subclasses
of \code{filler} (default \code{FALSE}).}

\item{...}{not used}
}
\value{
data.frame with columns \code{id} and \code{label}.
}
\description{
Find terms that have a given OWL someValuesFrom restriction
}
\examples{
goref <- semsql_connect(ontology = "go")
# cellular components that are part_of nucleus (GO:0005634)
find_by_restriction(goref, "BFO:0000050", "GO:0005634")
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{count_descendants}
\alias{count_descendants}
\title{Count the number of descendants of a term}
\usage{
count_descendants(x, term_id, predicate = "rdfs:subClassOf", ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{term_id}{character(1) CURIE.}

\item{predicate}{character(1) predicate to traverse
(default \code{"rdfs:subClassOf"}).}

\item{...}{not used}
}
\value{
integer(1).
}
\description{
Count the number of descendants of a term
}
\examples{
goref <- semsql_connect(ontology = "go")
count_descendants(goref, "GO:0006915") # all apoptosis subtypes
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bbop_sqlite_db_gz.R
\name{bbop_sqlite_db_gz}
\alias{bbop_sqlite_db_gz}
\title{retrieve and cache all filenames of Semantic SQL ontologies available, checking for updated
content relative to cache via ETag}
\usage{
bbop_sqlite_db_gz(
  url = "https://semanticsql.berkeleybop.io/",
  bfc = BiocFileCache::BiocFileCache(),
  rname = "bbop_sqlite_listing",
  timeout_connect = 5,
  timeout_total = 15
)
}
\arguments{
\item{url}{source address}

\item{bfc}{an instance of BiocFileCache}

\item{rname}{a string to use to identify the cached listing}

\item{timeout_connect}{passed to httr2::req_options connecttimeout parameter}

\item{timeout_total}{passed to httr2::req_options seconds parameter}
}
\value{
a character vector
}
\description{
retrieve and cache all filenames of Semantic SQL ontologies available, checking for updated
content relative to cache via ETag
}
\note{
There is no real etag discipline for the metadata, so full metadata
content is extracted on each call, digested, and compared to digest in cache.
Cache is updated when there is a discrepancy, except there is a guard against
rewriting cache with corrupted or zero-length data at endpoint.  Note this
code was negotiated at length with claude.ai Sonnet 4.6.
}
\examples{
gzs = bbop_sqlite_db_gz()
head(gzs)
length(gzs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{reconnect}
\alias{reconnect}
\title{Reconnect a SemsqlConn to its database}
\usage{
reconnect(x, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{...}{not used}
}
\value{
A new \code{SemsqlConn} object with an active connection.
}
\description{
Attempts to reconnect a disconnected \code{SemsqlConn} object to its
database. Returns a new \code{SemsqlConn}; the original cannot be modified
in place due to S7 value semantics.
}
\examples{
goref <- semsql_connect(ontology = "go")
disconnect(goref)
goref <- reconnect(goref)
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{is_connected}
\alias{is_connected}
\title{Test whether a SemsqlConn has a valid open connection}
\usage{
is_connected(x, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{...}{not used}
}
\value{
logical(1).
}
\description{
Test whether a SemsqlConn has a valid open connection
}
\examples{
goref <- semsql_connect(ontology = "go")
is_connected(goref) # TRUE
disconnect(goref)
is_connected(goref) # FALSE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{find_intersection}
\alias{find_intersection}
\title{Find terms that are descendants of a superclass and have a given restriction}
\usage{
find_intersection(x, superclass_id, relation_property, related_to_id, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{superclass_id}{character(1) CURIE of the superclass.}

\item{relation_property}{character(1) property CURIE for the restriction.}

\item{related_to_id}{character(1) filler CURIE for the restriction.}

\item{...}{not used}
}
\value{
data.frame with columns \code{id} and \code{label}.
}
\description{
Find terms that are descendants of a superclass and have a given restriction
}
\examples{
goref <- semsql_connect(ontology = "go")
# CC terms (GO:0005575) that are part_of nucleus (GO:0005634)
find_intersection(goref, "GO:0005575", "BFO:0000050", "GO:0005634")
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{search_labels}
\alias{search_labels}
\title{Search term labels in a SemsqlConn database}
\usage{
search_labels(x, pattern, limit = 20L)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{pattern}{character(1) substring to match against rdfs:label values
(SQL LIKE pattern, case-insensitive on most SQLite builds).}

\item{limit}{integer(1) maximum number of rows to return (default 20).}
}
\value{
data.frame with columns \code{subject} and \code{label}.
}
\description{
Search term labels in a SemsqlConn database
}
\examples{
goref <- semsql_connect(ontology = "go")
search_labels(goref, "apoptosis")
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{SemsqlConn}
\alias{SemsqlConn}
\title{SemsqlConn: S7 connection wrapper for SemanticSQL databases}
\usage{
SemsqlConn(con = NULL, db_path = character(0), ontology_prefix = character(0))
}
\arguments{
\item{con}{DBI connection object}

\item{db_path}{character path to SQLite database file}

\item{ontology_prefix}{character, e.g., 'CL' for cell ontology}
}
\value{
wrapped connection
}
\description{
An S7 class that encapsulates a SQLite connection to an ontology database
following the SemanticSQL schema. Provides methods for common ontology
queries including label lookup, ancestor/descendant traversal, and
relationship queries.

Properties:
\describe{
\item{con}{The DBI connection object (SQLiteConnection)}
\item{db_path}{character(1) path to the SQLite database file}
\item{ontology_prefix}{character(1) primary ontology prefix,
e.g. \code{"CL"} for the Cell Ontology}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{print}
\alias{print}
\title{Show method for SemsqlConn.
Concise one-line summary displayed when a \code{SemsqlConn} object is
auto-printed at the R prompt.}
\usage{
## S7 method for class <ontoProc2::SemsqlConn>
print(x, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{...}{not used}
}
\value{
x, invisibly
}
\description{
Show method for SemsqlConn.
Concise one-line summary displayed when a \code{SemsqlConn} object is
auto-printed at the R prompt.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cells_with_pmp.R
\name{cells_with_pmp}
\alias{cells_with_pmp}
\title{produce a table with cells exhibiting given proteins on plasma
membrane according to CL}
\usage{
cells_with_pmp(curies)
}
\arguments{
\item{curies}{a character vector in format "PR:nnnnnnnnn"}
}
\value{
a data.frame with columns cl, celltype, pr, protein
}
\description{
produce a table with cells exhibiting given proteins on plasma
membrane according to CL
}
\examples{
# 2064: microsialin, 1874: KLRB1-like protein
cells_with_pmp(c("PR:000002064", "PR:000001874"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{describe_table}
\alias{describe_table}
\title{Describe the columns of a table in a SemsqlConn database}
\usage{
describe_table(x, table_name, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{table_name}{character(1) name of the table.}

\item{...}{not used}
}
\value{
data.frame with PRAGMA table_info output (columns: cid, name, type,
notnull, dflt_value, pk).
}
\description{
Describe the columns of a table in a SemsqlConn database
}
\examples{
goref <- semsql_connect(ontology = "go")
describe_table(goref, "rdfs_label_statement")
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{get_synonyms}
\alias{get_synonyms}
\title{Get synonyms for a term}
\usage{
get_synonyms(
  x,
  term_id,
  type = c("all", "exact", "broad", "narrow", "related"),
  ...
)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{term_id}{character(1) CURIE.}

\item{type}{character(1) synonym scope: one of \code{"all"},
\code{"exact"}, \code{"broad"}, \code{"narrow"}, \code{"related"}.}

\item{...}{not used}
}
\value{
data.frame with columns \code{subject}, \code{predicate},
\code{synonym}.
}
\description{
Get synonyms for a term
}
\examples{
goref <- semsql_connect(ontology = "go")
get_synonyms(goref, "GO:0006915")
get_synonyms(goref, "GO:0006915", type = "exact")
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{get_term_info}
\alias{get_term_info}
\title{Retrieve a summary of information about a term}
\usage{
get_term_info(x, term_id, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{term_id}{character(1) CURIE.}

\item{...}{not used}
}
\value{
list with elements \code{id}, \code{label}, \code{definition},
\code{synonyms}, \code{superclasses}, \code{subclasses}.
}
\description{
Retrieve a summary of information about a term
}
\examples{
goref <- semsql_connect(ontology = "go")
info <- get_term_info(goref, "GO:0006915")
info$label
info$superclasses
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{get_ancestors}
\alias{get_ancestors}
\title{Get all ancestors of a term via entailed edges}
\usage{
get_ancestors(
  x,
  term_id,
  predicates = "rdfs:subClassOf",
  include_self = FALSE,
  ...
)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{term_id}{character(1) CURIE.}

\item{predicates}{character vector of predicate CURIEs to follow.
Defaults to \code{"rdfs:subClassOf"}. See \code{\link[=PREDICATES]{PREDICATES()}} for
common values.}

\item{include_self}{logical(1) whether to include the term itself
(default \code{FALSE}).}

\item{...}{not used}
}
\value{
data.frame with columns \code{id}, \code{label}, \code{predicate}.
}
\description{
Get all ancestors of a term via entailed edges
}
\examples{
goref <- semsql_connect(ontology = "go")
get_ancestors(goref, "GO:0006915")
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{report}
\alias{report}
\alias{report,ontoProc2::SemsqlConn-method}
\title{Display a detailed report of a SemsqlConn object}
\usage{
report(object, ...)

## S7 method for class <ontoProc2::SemsqlConn>
report(object, ...)
}
\arguments{
\item{object}{A \code{SemsqlConn} object.}

\item{...}{additional arguments (currently unused)}
}
\value{
The \code{SemsqlConn} object invisibly.
}
\description{
Displays a verbose, formatted representation of a \code{SemsqlConn} object
including connection status, database statistics (labeled terms, edges,
definitions), prefix breakdown, and available key tables. More informative
than \code{print()}, intended for interactive exploration.
}
\examples{
goref <- semsql_connect(ontology = "go")
report(goref)
disconnect(goref)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SemsqlConn-S7.R
\name{disconnect}
\alias{disconnect}
\title{Disconnect a SemsqlConn from its database}
\usage{
disconnect(x, quiet = FALSE, ...)
}
\arguments{
\item{x}{A \code{SemsqlConn} object.}

\item{quiet}{logical(1) if TRUE suppresses the disconnection message
(default FALSE).}

\item{...}{not used}
}
\value{
The \code{SemsqlConn} object invisibly.
}
\description{
Disconnect a SemsqlConn from its database
}
